from __future__ import annotations

import json
import os
import unittest
import warnings
from pathlib import Path
from typing import Any

import yaml

from checkov.common.checks_infra.checks_parser import GraphCheckParser
from checkov.common.checks_infra.registry import Registry
from checkov.common.models.enums import CheckResult
from checkov.runner_filter import RunnerFilter
from checkov.terraform import checks
from checkov.terraform.runner import Runner


class TestYamlPolicies(unittest.TestCase):
    def setUp(self) -> None:
        warnings.filterwarnings("ignore", category=ResourceWarning)
        warnings.filterwarnings("ignore", category=DeprecationWarning)

    def test_DatabricksWorkspaceDBFSRootCustomerManagedKey(self):
        self.go("DatabricksWorkspaceDBFSRootEncryptedWithCustomerManagedKey")

    def test_KmsKeyPolicyIsDefined(self):
        self.go("KmsKeyPolicyIsDefined")

    def test_NetworkFirewallHasLogging(self):
        self.go("NetworkFirewallHasLogging")

    def test_SecretsAreRotated(self):
        self.go("SecretsAreRotated")

    def test_S3BucketLifecycle(self):
        self.go("S3BucketLifecycle")

    def test_AccessControlGroupRuleDefine(self):
        self.go("AccessControlGroupRuleDefine")

    def test_S3BucketEventNotifications(self):
        self.go("S3BucketEventNotifications")

    def test_ADORepositoryHasMinTwoReviewers(self):
        self.go("ADORepositoryHasMinTwoReviewers")

    def test_VPCPeeringRouteTableOverlyPermissive(self):
        self.go("VPCPeeringRouteTableOverlyPermissive")

    def test_S3NotAllowAccessToAllAuthenticatedUsers(self):
        self.go("S3NotAllowAccessToAllAuthenticatedUsers")

    def test_CloudFrontHasCustomSSLCertificate(self):
        self.go("CloudFrontHasCustomSSLCertificate")

    def test_CodecommitApprovalRulesAttached(self):
        self.go("CodecommitApprovalRulesAttached")

    def test_RepositoryHasBranchProtection(self):
        self.go("RepositoryHasBranchProtection")

    def test_VPCHasFlowLog(self):
        self.go("VPCHasFlowLog")

    def test_APIGatewayEndpointsUsesCertificateForAuthentication(self):
        self.go("APIGatewayEndpointsUsesCertificateForAuthentication")

    def test_APIGatewayRequestParameterValidationEnabled(self):
        self.go("APIGatewayRequestParameterValidationEnabled")

    def test_CloudFrontUsesSecureProtocolsForHTTPS(self):
        self.go("CloudFrontUsesSecureProtocolsForHTTPS")

    def test_EMRClusterHasSecurityConfiguration(self):
        self.go("EMRClusterHasSecurityConfiguration")

    def test_OpenSearchDomainHasFineGrainedControl(self):
        self.go("OpenSearchDomainHasFineGrainedControl")

    def test_VPCHasRestrictedSG(self):
        self.go("VPCHasRestrictedSG")

    def test_APIGWLoggingLevelsDefinedProperly(self):
        self.go("APIGWLoggingLevelsDefinedProperly")

    def test_GuardDutyIsEnabled(self):
        self.go("GuardDutyIsEnabled")

    def test_SGAttachedToResource(self):
        self.go("SGAttachedToResource")

    def test_EC2InstanceHasIAMRoleAttached(self):
        self.go("EC2InstanceHasIAMRoleAttached")

    def test_StorageContainerActivityLogsNotPublic(self):
        self.go("StorageContainerActivityLogsNotPublic")

    def test_StorageCriticalDataEncryptedCMK(self):
        self.go("StorageCriticalDataEncryptedCMK")

    def test_VAconfiguredToSendReports(self):
        self.go("VAconfiguredToSendReports")

    def test_AWSConfigRecorderEnabled(self):
        self.go("AWSConfigRecorderEnabled")

    def test_CLoudFrontS3OriginConfigWithOAI(self):
        self.go("CLoudFrontS3OriginConfigWithOAI")

    def test_CloudFrontWebACLConfiguredWIthLog4jVulnerability(self):
        self.go("CloudFrontWebACLConfiguredWIthLog4jVulnerability")

    def test_ConfigRecorderRecordsAllGlobalResources(self):
        self.go("ConfigRecorderRecordsAllGlobalResources")

    def test_DMSEndpointHaveSSLConfigured(self):
        self.go("DMSEndpointHaveSSLConfigured")

    def test_ElastiCacheRedisConfiguredAutomaticFailOver(self):
        self.go("ElastiCacheRedisConfiguredAutomaticFailOver")

    def test_VAconfiguredToSendReportsToAdmins(self):
        self.go("VAconfiguredToSendReportsToAdmins")

    def test_VAisEnabledInStorageAccount(self):
        self.go("VAisEnabledInStorageAccount")

    def test_VAsetPeriodicScansOnSQL(self):
        self.go("VAsetPeriodicScansOnSQL")

    def test_CloudFrontHasResponseHeadersPolicy(self):
        self.go("CloudFrontHasResponseHeadersPolicy")

    def test_CloudtrailHasCloudwatch(self):
        self.go("CloudtrailHasCloudwatch")

    def test_S3BucketHasPublicAccessBlock(self):
        self.go("S3BucketHasPublicAccessBlock")

    def test_AccessToPostgreSQLFromAzureServicesIsDisabled(self):
        self.go("AccessToPostgreSQLFromAzureServicesIsDisabled")

    def test_AzureActiveDirectoryAdminIsConfigured(self):
        self.go("AzureActiveDirectoryAdminIsConfigured")

    def test_DisableAccessToSqlDBInstanceForRootUsersWithoutPassword(self):
        self.go("DisableAccessToSqlDBInstanceForRootUsersWithoutPassword")

    def test_GCPProjectHasNoLegacyNetworks(self):
        self.go("GCPProjectHasNoLegacyNetworks")

    def test_GCPDocumentAIProcessorEncryptedWithCMK(self):
        self.go("GCPDocumentAIProcessorEncryptedWithCMK")

    def test_GCPDocumentAIWarehouseLocationEncryptedWithCMK(self):
        self.go("GCPDocumentAIWarehouseLocationEncryptedWithCMK")

    def test_GCPVertexInstanceEncryptedWithCMK(self):
        self.go("GCPVertexInstanceEncryptedWithCMK")

    def test_GCPVertexAIEndpointEncryptedWithCMK(self):
        self.go("GCPVertexAIEndpointEncryptedWithCMK")

    def test_GCPVertexAIFeaturestoreEncryptedWithCMK(self):
        self.go("GCPVertexAIFeaturestoreEncryptedWithCMK")

    def test_GCPVertexAITensorboardEncryptedWithCMK(self):
        self.go("GCPVertexAITensorboardEncryptedWithCMK")

    def test_GCPVertexWorkbenchInstanceEncryptedWithCMK(self):
        self.go("GCPVertexWorkbenchInstanceEncryptedWithCMK")

    def test_GCPVertexWorkbenchInstanceNoPublicIp(self):
        self.go("GCPVertexWorkbenchInstanceNoPublicIp")

    def test_GCRContainerVulnerabilityScanningEnabled(self):
        self.go("GCRContainerVulnerabilityScanningEnabled")

    def test_AzureDataFactoriesEncryptedWithCustomerManagedKey(self):
        self.go("AzureDataFactoriesEncryptedWithCustomerManagedKey")

    def test_AzureUnattachedDisksAreEncrypted(self):
        self.go("AzureUnattachedDisksAreEncrypted")

    def test_AzureNetworkInterfacePublicIPAddressId(self):
        self.go("AzureNetworkInterfacePublicIPAddressId")

    def test_AzureAntimalwareIsConfiguredWithAutoUpdatesForVMs(self):
        self.go("AzureAntimalwareIsConfiguredWithAutoUpdatesForVMs")

    def test_ALBRedirectsHTTPToHTTPS(self):
        self.go("ALBRedirectsHTTPToHTTPS")

    def test_GCPLogBucketsConfiguredUsingLock(self):
        self.go("GCPLogBucketsConfiguredUsingLock")

    def test_CloudFunctionSecureHTTPTrigger(self):
        self.go("CloudFunctionSecureHTTPTrigger")

    def test_GCPAuditLogsConfiguredForAllServicesAndUsers(self):
        self.go("GCPAuditLogsConfiguredForAllServicesAndUsers")

    def test_GCPKMSCryptoKeysAreNotPubliclyAccessible(self):
        self.go("GCPKMSCryptoKeysAreNotPubliclyAccessible")

    def test_VirtualMachinesUtilizingManagedDisks(self):
        self.go("VirtualMachinesUtilizingManagedDisks")

    def test_RDSClusterHasBackupPlan(self):
        self.go("RDSClusterHasBackupPlan")

    def test_EBSAddedBackup(self):
        self.go("EBSAddedBackup")

    def test_AMRClustersNotOpenToInternet(self):
        self.go("AMRClustersNotOpenToInternet")

    def test_AutoScallingEnabledELB(self):
        self.go("AutoScallingEnabledELB")

    def test_AutoScalingEnabledLB(self):
        self.go("AutoScalingEnabledLB")

    def test_IAMGroupHasAtLeastOneUser(self):
        self.go("IAMGroupHasAtLeastOneUser")

    def test_IAMUserHasNoConsoleAccess(self):
        self.go("IAMUserHasNoConsoleAccess")

    def test_IAMUsersAreMembersAtLeastOneGroup(self):
        self.go("IAMUsersAreMembersAtLeastOneGroup")

    def test_IAMPolicyNotAllowFullIAMAccess(self):
        self.go("IAMPolicyNotAllowFullIAMAccess")

    def test_DataExplorerEncryptionUsesCustomKey(self):
        self.go("DataExplorerEncryptionUsesCustomKey")

    def test_MSQLenablesCustomerManagedKey(self):
        self.go("MSQLenablesCustomerManagedKey")

    def test_PGSQLenablesCustomerManagedKey(self):
        self.go("PGSQLenablesCustomerManagedKey")

    def test_StorageLoggingIsEnabledForBlobService(self):
        self.go("StorageLoggingIsEnabledForBlobService")

    def test_StorageLoggingIsEnabledForTableService(self):
        self.go("StorageLoggingIsEnabledForTableService")

    def test_SynapseLogMonitoringEnabledForSQLPool(self):
        self.go("SynapseLogMonitoringEnabledForSQLPool")
        
    def test_SynapseSQLPoolHasSecurityAlertPolicy(self):
        self.go("SynapseSQLPoolHasSecurityAlertPolicy")

    def test_SynapseSQLPoolHasVulnerabilityAssessment(self):
        self.go("SynapseSQLPoolHasVulnerabilityAssessment")

    def test_SynapseWorkspaceHasExtendedAuditLogs(self):
        self.go("SynapseWorkspaceHasExtendedAuditLogs")

    def test_VMHasBackUpMachine(self):
        self.go("VMHasBackUpMachine")

    def test_SubnetHasACL(self):
        self.go("SubnetHasACL")

    def test_GKEClustersAreNotUsingDefaultServiceAccount(self):
        self.go("GKEClustersAreNotUsingDefaultServiceAccount")

    def test_AzureMSSQLServerHasSecurityAlertPolicy(self):
        self.go("AzureMSSQLServerHasSecurityAlertPolicy")

    def test_AzureSynapseWorkspacesHaveNoIPFirewallRulesAttached(self):
        self.go("AzureSynapseWorkspacesHaveNoIPFirewallRulesAttached")

    def test_EncryptedEBSVolumeOnlyConnectedToEC2s(self):
        self.go("EncryptedEBSVolumeOnlyConnectedToEC2s")

    def test_ServiceAccountHasGCPmanagedKey(self):
        self.go("ServiceAccountHasGCPmanagedKey")

    def test_AutoScalingEnableOnDynamoDBTables(self):
        self.go("AutoScalingEnableOnDynamoDBTables")

    def test_EIPAllocatedToVPCAttachedEC2(self):
        self.go("EIPAllocatedToVPCAttachedEC2")

    def test_EFSAddedBackup(self):
        self.go("EFSAddedBackup")

    def test_EFSAddedBackupSuppress(self):
        self.go("EFSAddedBackupSuppress", "EFSAddedBackup")

    def test_Route53ARecordAttachedResource(self):
        self.go("Route53ARecordAttachedResource")

    def test_RouteTablePublicSubnetConnection(self):
        self.go("RouteTablePublicSubnetConnection")

    def test_PostgresRDSHasQueryLoggingEnabled(self):
        self.go("PostgresRDSHasQueryLoggingEnabled")

    def test_HTTPNotSendingPasswords(self):
        self.go("HTTPNotSendingPasswords")

    def test_PostgresDBHasQueryLoggingEnabled(self):
        self.go("PostgresDBHasQueryLoggingEnabled")

    def test_ALBProtectedByWAF(self):
        self.go("ALBProtectedByWAF")

    def test_APIProtectedByWAF(self):
        self.go("APIProtectedByWAF")

    def test_SQLServerAuditingEnabled(self):
        self.go("SQLServerAuditingEnabled")

    def test_WAF2HasLogs(self):
        self.go("WAF2HasLogs")

    def test_AppSyncProtectedByWAF(self):
        self.go("AppSyncProtectedByWAF")

    def test_SQLServerAuditingRetention90Days(self):
        self.go("SQLServerAuditingRetention90Days")

    def test_AWSSSMParameterShouldBeEncrypted(self):
        self.go("AWSSSMParametershouldbeEncrypted", "AWSSSMParameterShouldBeEncrypted")

    def test_AWSNATGatewaysshouldbeutilized(self):
        self.go("AWSNATGatewaysshouldbeutilized")

    def test_GCPKMSKeyRingsAreNotPubliclyAccessible(self):
        self.go("GCPKMSKeyRingsAreNotPubliclyAccessible")

    def test_GCPContainerRegistryReposAreNotPubliclyAccessible(self):
        self.go("GCPContainerRegistryReposAreNotPubliclyAccessible")

    def test_GCPDialogFlowAgentLoggingEnabled(self):
        self.go("GCPDialogFlowAgentLoggingEnabled")

    def test_GCPDialogFlowCxAgentLoggingEnabled(self):
        self.go("GCPDialogFlowCxAgentLoggingEnabled")

    def test_GCPDialogFlowCxWebhookLoggingEnabled(self):
        self.go("GCPDialogFlowCxWebhookLoggingEnabled")

    def test_GCPVertexAIPrivateEndpoint(self):
        self.go("GCPVertexAIPrivateEndpoint")

    def test_GCPVertexAIPrivateIndexEndpoint(self):
        self.go("GCPVertexAIPrivateIndexEndpoint")

    def test_GCPTpuV2VmPrivateEndpoint(self):
        self.go("GCPTpuV2VmPrivateEndpoint")

    def test_GCPVertexRuntimePrivate(self):
        self.go("GCPVertexRuntimePrivate")

    def test_GCPVertexRuntimeEncryptedWithCMK(self):
        self.go("GCPVertexRuntimeEncryptedWithCMK")

    def test_S3BucketVersioning(self):
        self.go("S3BucketVersioning")

    def test_S3PublicACLRead(self):
        self.go("S3PublicACLRead")

    def test_S3PublicACLWrite(self):
        self.go("S3PublicACLWrite")

    def test_S3BucketEncryption(self):
        self.go("S3BucketEncryption")

    def test_S3BucketLogging(self):
        self.go("S3BucketLogging")

    def test_AdministratorUserNotAssociatedWithAPIKey(self):
        self.go("AdministratorUserNotAssociatedWithAPIKey")

    def test_ApplicationGatewayEnablesWAF(self):
        self.go("ApplicationGatewayEnablesWAF")

    def test_S3KMSEncryptedByDefault(self):
        self.go("S3KMSEncryptedByDefault")

    def test_GCPNetworkDoesNotUseDefaultFirewall(self):
        self.go("GCPNetworkDoesNotUseDefaultFirewall")

    def test_S3BucketReplicationConfiguration(self):
        self.go("S3BucketReplicationConfiguration")

    def test_AppLoadBalancerTLS12(self):
        self.go("AppLoadBalancerTLS12")

    def test_GCPPostgreSQLDatabaseFlaglog_durationIsSetToON(self):
        self.go("GCPPostgreSQLDatabaseFlaglog_durationIsSetToON")

    def test_GCPPostgreSQLDatabaseFlaglog_executor_statsIsSetToOFF(self):
        self.go("GCPPostgreSQLDatabaseFlaglog_executor_statsIsSetToOFF")

    def test_GCPPostgreSQLDatabaseFlaglog_parser_statsIsSetToOFF(self):
        self.go("GCPPostgreSQLDatabaseFlaglog_parser_statsIsSetToOFF")

    def test_GCPPostgreSQLDatabaseFlaglog_planner_statsIsSetToOFF(self):
        self.go("GCPPostgreSQLDatabaseFlaglog_planner_statsIsSetToOFF")

    def test_GCPPostgreSQLDatabaseFlaglog_statement_statsIsSetToOFF(self):
        self.go("GCPPostgreSQLDatabaseFlaglog_statement_statsIsSetToOFF")

    def test_GCPComputeFirewallOverlyPermissiveToAllTraffic(self):
        self.go("GCPComputeFirewallOverlyPermissiveToAllTraffic")

    def test_AzureConfigMSSQLwithAD(self):
        self.go("AzureConfigMSSQLwithAD")

    def test_AzurePostgreSQLFlexServerNotOverlyPermissive(self):
        self.go("AzurePostgreSQLFlexServerNotOverlyPermissive")

    def test_GCPMySQLdbInstancePoint_In_TimeRecoveryBackupIsEnabled(self):
        self.go("GCPMySQLdbInstancePoint_In_TimeRecoveryBackupIsEnabled")

    def test_GCPdisableAlphaClusterFeatureInKubernetesEngineClusters(self):
        self.go("GCPdisableAlphaClusterFeatureInKubernetesEngineClusters")

    def test_AzureContainerInstanceconfigManagedIdentity(self):
            self.go("AzureContainerInstanceconfigManagedIdentity")

    def test_AzureAKSclusterAzureCNIEnabled(self):
        self.go("AzureAKSclusterAzureCNIEnabled")

    def test_AzureACR_HTTPSwebhook(self):
        self.go("AzureACR_HTTPSwebhook")

    def test_AzureSubnetConfigWithNSG(self):
        self.go("AzureSubnetConfigWithNSG")

    def test_AzureKeyVaultConfigPrivateEndpoint(self):
        self.go("AzureKeyVaultConfigPrivateEndpoint")

    def test_AzureStorageAccConfigWithPrivateEndpoint(self):
        self.go("AzureStorageAccConfigWithPrivateEndpoint")

    def test_OCI_K8EngineClusterBootVolConfigInTransitEncryption(self):
            self.go("OCI_K8EngineClusterBootVolConfigInTransitEncryption")

    def test_OCI_K8EngineClusterPodSecPolicyEnforced(self):
            self.go("OCI_K8EngineClusterPodSecPolicyEnforced")

    def test_OCI_KubernetesEngineClusterEndpointConfigWithNSG(self):
            self.go("OCI_KubernetesEngineClusterEndpointConfigWithNSG")

    def test_OCI_NFSaccessRestrictedToRootUsers(self):
            self.go("OCI_NFSaccessRestrictedToRootUsers")

    def test_OCI_NSGNotAllowRDP(self):
            self.go("OCI_NSGNotAllowRDP")

    def test_AzureSQLserverNotOverlyPermissive(self):
            self.go("AzureSQLserverNotOverlyPermissive")

    def test_AzureRecoveryServicesvaultConfigManagedIdentity(self):
            self.go("AzureRecoveryServicesvaultConfigManagedIdentity")

    def test_AzureAutomationAccConfigManagedIdentity(self):
            self.go("AzureAutomationAccConfigManagedIdentity")

    def test_AzureMariaDBserverUsingTLS_1_2(self):
            self.go("AzureMariaDBserverUsingTLS_1_2")
   
    def test_AzureMLWorkspaceWithPublicStorageAccount(self):
            self.go("AzureMLWorkspaceHBIPublicNetwork")

    def test_AzureMLWorkspacePublicNetwork(self):
            self.go("AzureMLWorkspacePublicNetwork")

    def test_AzureStorageAccountEnableSoftDelete(self):
            self.go("AzureStorageAccountEnableSoftDelete")

    def test_AWSdisableS3ACL(self):
        self.go("AWSdisableS3ACL")

    def test_AWS_private_MWAA_environment(self):
        self.go("AWS_private_MWAA_environment")

    def test_AzureStorageAccConfigSharedKeyAuth(self):
        self.go("AzureStorageAccConfigSharedKeyAuth")

    def test_AzureStorageAccConfig_SAS_expirePolicy(self):
        self.go("AzureStorageAccConfig_SAS_expirePolicy")

    def test_AzureVMconfigPublicIP_SerialConsoleAccess(self):
        self.go("AzureVMconfigPublicIP_SerialConsoleAccess")

    def test_AzurePostgreSQLserverConfigPrivEndpt(self):
        self.go("AzurePostgreSQLserverConfigPrivEndpt")

    def test_AzureMariaDBserverConfigPrivEndpt(self):
        self.go("AzureMariaDBserverConfigPrivEndpt")

    def test_AzureMySQLserverConfigPrivEndpt(self):
        self.go("AzureMySQLserverConfigPrivEndpt")

    def test_AzureMSSQLserverConfigPrivEndpt(self):
        self.go("AzureMSSQLserverConfigPrivEndpt")

    def test_AzureSynapseWorkspaceVAisEnabled(self):
        self.go("AzureSynapseWorkspaceVAisEnabled")

    def test_IBM_LoadBalancerforVPCisPrivate(self):
        self.go("IBM_LoadBalancerforVPCisPrivate")

    def test_IBM_VPCclassicAccessIsDisabled(self):
        self.go("IBM_VPCclassicAccessIsDisabled")

    def test_SageMakerIAMPolicyOverlyPermissiveToAllTraffic(self):
        self.go("SageMakerIAMPolicyOverlyPermissiveToAllTraffic")

    def test_IBM_RestrictServiceIDCreationInAccountSettings(self):
        self.go("IBM_RestrictServiceIDCreationInAccountSettings")

    def test_IBM_EnableMFAatAccountLevel(self):
        self.go("IBM_EnableMFAatAccountLevel")

    def test_IBM_RestrictAPIkeyCreationInAccountSettings(self):
        self.go("IBM_RestrictAPIkeyCreationInAccountSettings")

    def test_IBM_K8sClustersAccessibleViaPrivateEndPt(self):
        self.go("IBM_K8sClustersAccessibleViaPrivateEndPt")

    def test_RDSEncryptionInTransit(self):
        self.go("RDSEncryptionInTransit")

    def test_ACMWildcardDomainName(self):
        self.go("ACMWildcardDomainName")

    def test_CloudfrontOriginNotHTTPSOnly(self):
        self.go("CloudfrontOriginNotHTTPSOnly")

    def test_SQSEncryptionCMK(self):
        self.go("SQSEncryptionCMK")

    def test_LBTargetGroup(self):
        self.go("LBTargetGroup")

    def test_AzureSpringCloudTLSDisabled(self):
        self.go("AzureSpringCloudTLSDisabled")

    def test_GCPComputeRegionalForwardingRuleCheck(self):
        self.go("GCPComputeRegionalForwardingRuleCheck")

    def test_GCPComputeGlobalForwardingRuleCheck(self):
        self.go("GCPComputeGlobalForwardingRuleCheck")

    def test_AzureMySQLFlexibleServerConfigPrivEndpt(self):
        self.go("AzureMySQLFlexibleServerConfigPrivEndpt")

    def test_AzurePostgreSQLFlexibleServerConfigPrivEndpt(self):
        self.go("AzurePostgreSQLFlexibleServerConfigPrivEndpt")

    def test_OSSBucketPublic(self):
        self.go("OSSBucketPublic")

    def test_Route53ZoneHasMatchingQueryLog(self):
        self.go("Route53ZoneHasMatchingQueryLog")

    def test_ALBWebACLConfiguredWIthLog4jVulnerability(self):
        self.go("ALBWebACLConfiguredWIthLog4jVulnerability")

    def test_APIGatewayWebACLConfiguredWIthLog4jVulnerability(self):
        self.go("APIGatewayWebACLConfiguredWIthLog4jVulnerability")

    def test_AppsyncWebACLConfiguredWIthLog4jVulnerability(self):
        self.go("AppsyncWebACLConfiguredWIthLog4jVulnerability")

    def test_Route53ZoneEnableDNSSECSigning(self):
        self.go("Route53ZoneEnableDNSSECSigning")

    def test_LBWeakCiphers(self):
        self.go("LBWeakCiphers")

    def test_LambdaOpenCorsPolicy(self):
        self.go("LambdaOpenCorsPolicy")

    def test_registry_load(self):
        registry = Registry(parser=GraphCheckParser(), checks_dir=str(
            Path(__file__).parent.parent.parent.parent.parent / "checkov" / "terraform" / "checks" / "graph_checks"))
        registry.load_checks()
        self.assertGreater(len(registry.checks), 0)

    def go(self, dir_name: str, check_name: str | None = None) -> None:
        dir_path = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                                f"resources/{dir_name}")
        check_name = dir_name if check_name is None else check_name
        assert os.path.exists(dir_path)
        policy_dir_path = os.path.dirname(checks.__file__)
        assert os.path.exists(policy_dir_path)
        found = False
        for root, d_names, f_names in os.walk(policy_dir_path):
            for f_name in f_names:
                if f_name == f"{check_name}.yaml":
                    found = True
                    policy = load_yaml_data(f_name, root)
                    assert policy is not None
                    expected = load_yaml_data("expected.yaml", dir_path)
                    assert expected is not None
                    report = get_policy_results(dir_path, [policy['metadata']['id']])

                    expected_to_fail = expected.get('fail', [])
                    expected_to_pass = expected.get('pass', [])
                    expected_to_skip = expected.get('skip', [])

                    self.assert_entities(expected_to_pass, report.passed_checks, True)
                    self.assert_entities(expected_to_fail, report.failed_checks, False)
                    self.assert_entities(expected_to_skip, report.skipped_checks, True)

        assert found

    def assert_entities(self, expected_entities: list[str], results: list[CheckResult], assertion: bool) -> None:
        self.assertEqual(len(expected_entities), len(results),
                         f"mismatch in number of results in {'passed' if assertion else 'failed'}, "
                         f"expected: {len(expected_entities)}, got: {len(results)}")
        for expected_entity in expected_entities:
            found = False
            for check_result in results:
                entity_id = check_result.resource
                if entity_id == expected_entity:
                    found = True
                    break
            self.assertTrue(found, f"expected to find entity {expected_entity}, {'passed' if assertion else 'failed'}")


def get_policy_results(root_folder: str, check_ids: list[str], external_registries=None):
    graph_runner = Runner()
    graph_runner.external_registries = external_registries if external_registries else []
    report = graph_runner.run(root_folder, runner_filter=RunnerFilter(checks=check_ids))
    return report


def wrap_policy(policy):
    policy['query'] = policy['definition']
    del policy['definition']


def load_yaml_data(source_file_name: str, dir_path: str) -> dict[str, Any] | None:
    expected_path = os.path.join(dir_path, source_file_name)
    if not os.path.exists(expected_path):
        return None

    with open(expected_path, "r") as f:
        expected_data = yaml.safe_load(f)

    return json.loads(json.dumps(expected_data))



import unittest
from pathlib import Path

from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.module.registry import module_registry
from checkov.terraform.runner import Runner


class TestModuleCheck(unittest.TestCase):
    def test_module_version(self):
        external_checks = Path.joinpath(Path(__file__).parent,
                                        "example_external_dir_with_module_version_check/extra_checks").as_posix()
        test_files_dir = Path(__file__).parent / "resources"

        report = Runner().run(root_folder=test_files_dir, runner_filter=RunnerFilter(checks=["CKV_TF_MODULE_1"]),
                              external_checks_dir=[external_checks])
        summary = report.get_summary()

        self.assertEqual(summary["passed"], 2)
        self.assertEqual(summary["failed"], 3)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        # remove custom checks
        check = next(c for c in module_registry.checks["module"] if c.id == "CKV_TF_MODULE_1")
        module_registry.checks["module"].remove(check)
        check = next(c for c in module_registry.checks["module"] if c.id == "CKV_TF_MODULE_2")
        module_registry.checks["module"].remove(check)

    def test_immutable_module(self):
        external_checks = Path.joinpath(Path(__file__).parent,
                                        "example_external_dir_with_module_version_check/extra_checks").as_posix()
        test_files_dir = Path(__file__).parent / "resources"

        report = Runner().run(root_folder=test_files_dir, runner_filter=RunnerFilter(checks=["CKV_TF_MODULE_2"]),
                              external_checks_dir=[external_checks])
        summary = report.get_summary()

        self.assertEqual(summary["passed"], 1)
        self.assertEqual(summary["failed"], 4)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        # remove custom checks
        check = next(c for c in module_registry.checks["module"] if c.id == "CKV_TF_MODULE_1")
        module_registry.checks["module"].remove(check)
        check = next(c for c in module_registry.checks["module"] if c.id == "CKV_TF_MODULE_2")
        module_registry.checks["module"].remove(check)


if __name__ == "__main__":
    unittest.main()



"""Trailer wire format: ``# checkov-digest: <hex-DER-ECDSA-P256-SHA256>\\n``."""
from __future__ import annotations

import string


TRAILER_PREFIX = b"# checkov-digest: "

# P-256 DER signatures are 63-72 bytes (SEC 1 §C.5 / RFC 5480 §2.1) → 126-144 hex chars.
_HEX_MIN = 126
_HEX_MAX = 144

_LOWERCASE_HEX_BYTES = frozenset((string.digits + "abcdef").encode("ascii"))


def _split_last_line(file_bytes: bytes) -> "tuple[bytes, int, bytes] | None":
    if not file_bytes or not file_bytes.endswith(b"\n"):
        return None
    body = file_bytes[:-1]
    last_nl = body.rfind(b"\n")
    return body, last_nl, body[last_nl + 1:]


def parse_trailer(file_bytes: bytes) -> "tuple[bytes, bytes] | None":
    split = _split_last_line(file_bytes)
    if split is None:
        return None
    _body, last_nl, trailer_line = split

    if not trailer_line.startswith(TRAILER_PREFIX):
        return None

    hex_payload = trailer_line[len(TRAILER_PREFIX):]
    if not (_HEX_MIN <= len(hex_payload) <= _HEX_MAX):
        return None
    if not all(b in _LOWERCASE_HEX_BYTES for b in hex_payload):
        return None

    # last_nl == -1 → file is the trailer line only → empty signed body.
    signed_bytes = file_bytes[:last_nl + 1] if last_nl >= 0 else b""
    return signed_bytes, hex_payload


def has_trailer_prefix(file_bytes: bytes) -> bool:
    split = _split_last_line(file_bytes)
    if split is None:
        return False
    return split[2].startswith(TRAILER_PREFIX)


def has_double_trailer(file_bytes: bytes) -> bool:
    """True iff the last TWO lines both start with the trailer prefix."""
    split = _split_last_line(file_bytes)
    if split is None:
        return False
    body, last_nl, last_line = split
    if last_nl < 0:
        return False
    if not last_line.startswith(TRAILER_PREFIX):
        return False
    body_without_last = body[:last_nl]
    prev_nl = body_without_last.rfind(b"\n")
    return body_without_last[prev_nl + 1:].startswith(TRAILER_PREFIX)


def decode_hex_signature(hex_payload: bytes) -> "bytes | None":
    try:
        return bytes.fromhex(hex_payload.decode("ascii"))
    except ValueError:
        # UnicodeDecodeError is a subclass of ValueError, so this catches both
        # non-ASCII payloads and non-hex characters in one branch.
        return None


__all__ = [
    "TRAILER_PREFIX",
    "decode_hex_signature",
    "has_double_trailer",
    "has_trailer_prefix",
    "parse_trailer",
]



from __future__ import annotations

import os
import unittest
from typing import Any, Iterable

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.integration_features.features.fixes_integration import FixesIntegration
from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import integration as metadata_integration
from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
from checkov.common.models.enums import CheckResult
from checkov.common.output.record import Record
from checkov.common.output.report import Report


_old_check_metadata = None


class TestFixesIntegration(unittest.TestCase):
    def test_integration_valid(self):
        instance = BcPlatformIntegration()
        instance.skip_fixes = False
        instance.platform_integration_configured = True
        instance.on_prem = False

        fixes_integration = FixesIntegration(instance)

        self.assertTrue(fixes_integration.is_valid())

        instance.on_prem = True
        self.assertFalse(fixes_integration.is_valid())

        instance.on_prem = False
        instance.skip_fixes = True
        self.assertFalse(fixes_integration.is_valid())

        instance.platform_integration_configured = False
        self.assertFalse(fixes_integration.is_valid())

        instance.skip_fixes = False
        self.assertFalse(fixes_integration.is_valid())

        fixes_integration.integration_feature_failures = True
        self.assertFalse(fixes_integration.is_valid())

    def test_apply_fixes_to_report(self):
        instance = BcPlatformIntegration()
        instance.skip_fixes = False
        instance.platform_integration_configured = True

        fixes_integration = FixesIntegration(instance)
        fixes_integration._get_fixes_for_file = mock_fixes_response

        metadata_integration.check_metadata = {
            'custom_aws_12345': {'guideline': 'https://docs.bridgecrew.io/docs/ensure-vpc-subnets-do-not-assign-public-ip-by-default'},
            'CKV_AWS_130': {
                'id': 'BC_AWS_NETWORKING_53',
                'title': 'Ensure VPC subnets do not assign public IP by default',
                'guideline': 'https://docs.bridgecrew.io/docs/ensure-vpc-subnets-do-not-assign-public-ip-by-default',
                'severity': 'MEDIUM',
                'pcSeverity': 'MEDIUM',
                'category': 'Networking',
                'checkovId': 'CKV_AWS_130',
                'constructiveTitle': 'Ensure VPC subnets do not assign public IP by default',
                'descriptiveTitle': 'AWS VPC subnets should not allow automatic public IP assignment',
                'pcPolicyId': '11743cd3-35e4-4639-91e1-bc87b52d4cf5',
                'additionalPcPolicyIds': ['11743cd3-35e4-4639-91e1-bc87b52d4cf5'],
                'benchmarks': {}
            }
        }

        metadata_integration.bc_to_ckv_id_mapping = {
            'BC_AWS_NETWORKING_53': 'CKV_AWS_130'
        }

        report = Report(CheckType.TERRAFORM)

        file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'resources', 'main.tf')

        report.add_record(Record(
            check_id='CKV_AWS_130',
            bc_check_id='BC_AWS_NETWORKING_53',
            check_result={'result': CheckResult.FAILED, 'evaluated_keys': ['map_public_ip_on_launch']},
            check_name='Ensure VPC subnets do not assign public IP by default',
            check_class='checkov.terraform.checks.resource.aws.SubnetPublicIP',
            code_block=[(1, 'resource "aws_subnet" "s" {\n'), (2, '  map_public_ip_on_launch = true\n'), (3, '}\n')],
            evaluations=None,
            file_abs_path=file,
            file_line_range=[2, 3],
            file_path='/main.tf',
            resource='aws_subnet.s'
        ))

        report.add_record(Record(
            check_id='custom_aws_12345',
            bc_check_id='custom_aws_12345',
            check_result={'result': CheckResult.FAILED, 'evaluated_keys': ['map_public_ip_on_launch']},
            check_name='Cloned-Ensure VPC subnets do not assign public IP by default',
            check_class='checkov.terraform.checks.resource.aws.SubnetPublicIP',
            code_block=[(1, 'resource "aws_subnet" "s" {\n'), (2, '  map_public_ip_on_launch = true\n'), (3, '}\n')],
            evaluations=None,
            file_abs_path=file,
            file_line_range=[2, 3],
            file_path='/main.tf',
            resource='aws_subnet.s'
        ))

        fixes_integration.post_runner(report)

        self.assertTrue(all(r.fixed_definition is not None for r in report.failed_checks))

    def setUp(self) -> None:
        self._old_check_metadata = metadata_integration.check_metadata

    def tearDown(self) -> None:
        metadata_integration.check_metadata = self._old_check_metadata
        metadata_integration.bc_to_ckv_id_mapping = {}


def mock_fixes_response(check_type: str, filename: str, file_contents: str, failed_checks: Iterable[Record]
    ) -> dict[str, Any] | None:
    return {
        'filePath': '/private/tmp/custom/main.tf',
        'fixes': [
            {
                'resourceId': 'aws_subnet.s',
                'policyId': 'BC_AWS_NETWORKING_53',
                'originalStartLine': 1,
                'originalEndLine': 3,
                'fixedDefinition': 'resource "aws_subnet" "s" {\n}\n'
            },
            {
                'resourceId': 'aws_subnet.s',
                'policyId': 'custom_aws_12345',
                'originalStartLine': 1,
                'originalEndLine': 3,
                'fixedDefinition': 'resource "aws_subnet" "s" {\n}\n'
            }
        ]
    }


if __name__ == '__main__':
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_docdb as docdb

class DocDBStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define the DocDB Cluster Parameter Group
        db_parameter_group = docdb.CfnDBClusterParameterGroup(
            self,
            "DocDBClusterParameterGroup",
            description="Custom DocDB Cluster Parameter Group",
            family="docdb4.0",
            parameters={
                "audit_logs": "enabled",
            }
        )

app = core.App()
DocDBStack(app, "DocDBStack")
app.synth()



import unittest

import hcl2

from checkov.terraform.checks.resource.gcp.GoogleProjectDefaultNetwork import check
from checkov.common.models.enums import CheckResult


class TestGoogleProjectDefaultNetwork(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
                resource "google_project" "my_project" {
                  name       = "My Project"
                  project_id = "your-project-id"
                  org_id     = "1234567"
                }
                """)
        resource_conf = hcl_res['resource'][0]['google_project']['my_project']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
                        resource "google_project" "my_project" {
                          name       = "My Project"
                          project_id = "your-project-id"
                          org_id     = "1234567"
                          auto_create_network   = false
                        }
                        """)
        resource_conf = hcl_res['resource'][0]['google_project']['my_project']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.aws.EC2DetailedMonitoringEnabled import check
from checkov.common.models.enums import CheckResult


class TestEC2PublicIP(unittest.TestCase):

    def test_failure1(self):
        hcl_res = hcl2.loads("""
            resource "aws_instance" "test" {
            }
        """)
        resource_conf = hcl_res['resource'][0]['aws_instance']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure2(self):
        hcl_res = hcl2.loads("""
            resource "aws_launch_template" "test" {
                monitoring = false
            }
        """)
        resource_conf = hcl_res['resource'][0]['aws_launch_template']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_aws_instance(self):
        hcl_res = hcl2.loads("""
            resource "aws_instance" "test" {
                monitoring = true
            }
        """)
        resource_conf = hcl_res['resource'][0]['aws_instance']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest

from checkov.terraform.checks.resource.aws.CloudtrailLogValidation import check
from checkov.common.models.enums import CheckResult


class TestCloudtrailLogValidation(unittest.TestCase):

    def test_failure(self):
        resource_conf = {'enable_logging': [True], 's3_bucket_name': ['${foo}'],
                         'is_multi_region_trail': [True], 'include_global_service_events': [True],
                         'name': ['foo']}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {'enable_logging': [True], 's3_bucket_name': ['${foo}'], 'enable_log_file_validation': [True],
                         'is_multi_region_trail': [True], 'include_global_service_events': [True],
                         'kms_key_id': ['${foo}'], 'name': ['foo']}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from typing import List

from checkov.common.output.record import Record


def _filter_reports_for_incident_ids(failed_checks: List[Record], policy_names: List[str]) \
        -> List[Record]:
    return [failed_check for failed_check in failed_checks if failed_check.check_id in policy_names]



from pathlib import Path

from checkov.bicep.utils import get_scannable_file_paths


def test_get_scannable_file_paths(tmp_path: Path):
    # given
    (tmp_path / "storage.json").touch()
    (tmp_path / "storage.bicep").touch()

    (tmp_path / ".bicep").mkdir()
    (tmp_path / ".bicep/main.bicep").touch()

    # when
    file_paths = get_scannable_file_paths(root_folder=tmp_path)

    # then
    assert file_paths == {
        tmp_path / "storage.bicep",
        tmp_path / ".bicep/main.bicep",
    }



import os
import tempfile
import unittest
from unittest import mock

from checkov.serverless.parsers.parser \
    import process_variables, _tokenize_by_commas, _token_to_type_and_loc, _parse_var


IRRELEVANT_DIR = os.curdir

class TestParser(unittest.TestCase):

    #########################################
    # "Self" variable processing

    def test_self_simple(self):
        case = {
            "source": "var-data",
            "consumer": "${self:source}"
        }
        expected = {
            "source": "var-data",
            "consumer": "var-data"
        }
        self.assertEqual(expected, process_variables(case, IRRELEVANT_DIR))

    def test_self_with_default(self):
        case = {
            "consumer": "${self:source-of-var-data,aDefaultValue}"
        }
        expected = {
            "consumer": "aDefaultValue"
        }
        self.assertEqual(expected, process_variables(case, IRRELEVANT_DIR))

    def test_self_nexted(self):
        case = {
            "source": {
                "nested1": "value one",
                "more_nesting": {
                    "nested2": "value two"
                }
            },
            "consumer": "${self:source.nested1} - ${self:source.more_nesting.nested2} - ${self:bogus,aDefault}"
        }
        expected = {
            "source": {
                "nested1": "value one",
                "more_nesting": {
                    "nested2": "value two"
                }
            },
            "consumer": "value one - value two - aDefault"
        }
        self.assertEqual(expected, process_variables(case, IRRELEVANT_DIR))

    def test_self_invalid(self):
        case = {
            "consumer": "${self:bogus-no-default}"
        }
        expected = {
            "consumer": "${self:bogus-no-default}"
        }
        self.assertEqual(expected, process_variables(case, IRRELEVANT_DIR))

    def test_self_list(self):
        case = {
            "source": "var-data",
            "consumer-list": [
                {
                    "consumer": "${self:source}"
                }
            ]
        }
        expected = {
            "source": "var-data",
            "consumer-list": [
                {
                    "consumer": "var-data"
                }
            ]
        }
        self.assertEqual(expected, process_variables(case, IRRELEVANT_DIR))

    def test_self_real_world_data(self):
        case = {
            'service': 'my-sls-service',
            'custom': {
                'my_sneaky_custom_var': '*',
                '__startline__': 4,
                '__endline__': 6
            },
            'provider': {
                'name': 'aws',
                'runtime': 'python3.7',
                'stackName': 'lambda-',
                'tag': '${opt:tag}',
                'iamRoleStatements': [
                    {
                        'Effect': 'Allow',
                        'Action': '${self:custom.my_sneaky_custom_var}',
                        'Resource': '${self:custom.my_sneaky_custom_var}',
                        '__startline__': 13,
                        '__endline__': 17
                    }
                ],
                '__startline__': 7,
                '__endline__': 17
            },
            'functions': {
                'myFunc': {
                    'name': 'myFunc-provider-level-with-var',
                    'tags': {
                        'RESOURCE': 'lambda',
                        'PUBLIC': False,
                        '__startline__': 21,
                        '__endline__': 23
                    },
                    'iamRoleStatements': [
                        {
                            'Effect': 'Allow',
                            'Action': ['lambda:InvokeFunction'],
                            'Resource': [
                                'arn:aws:lambda:#{AWS::Region}:#{AWS::AccountId}:function:invokedLambda'],
                            '__startline__': 24,
                            '__endline__': 29
                        }
                    ],
                    'handler': 'Handler.handle',
                    'timeout': 600,
                    'memorySize': 320,
                    '__startline__': 19,
                    '__endline__': 31
                },
                '__startline__': 18,
                '__endline__': 31
            },
            '__startline__': 1,
            '__endline__': 31
        }
        expected = {
            'service': 'my-sls-service',
            'custom': {
                'my_sneaky_custom_var': '*',
                '__startline__': 4,
                '__endline__': 6
            },
            'provider': {
                'name': 'aws',
                'runtime': 'python3.7',
                'stackName': 'lambda-',
                'tag': '${opt:tag}',
                'iamRoleStatements': [
                    {
                        'Effect': 'Allow',
                        'Action': '*',
                        'Resource': '*',
                        '__startline__': 13,
                        '__endline__': 17
                    }
                ],
                '__startline__': 7,
                '__endline__': 17
            },
            'functions': {
                'myFunc': {
                    'name': 'myFunc-provider-level-with-var',
                    'tags': {
                        'RESOURCE': 'lambda',
                        'PUBLIC': False,
                        '__startline__': 21,
                        '__endline__': 23
                    },
                    'iamRoleStatements': [
                        {
                            'Effect': 'Allow',
                            'Action': ['lambda:InvokeFunction'],
                            'Resource': [
                                'arn:aws:lambda:#{AWS::Region}:#{AWS::AccountId}:function:invokedLambda'],
                            '__startline__': 24,
                            '__endline__': 29
                        }
                    ],
                    'handler': 'Handler.handle',
                    'timeout': 600,
                    'memorySize': 320,
                    '__startline__': 19,
                    '__endline__': 31
                },
                '__startline__': 18,
                '__endline__': 31
            },
            '__startline__': 1,
            '__endline__': 31
        }
        self.assertEqual(expected, process_variables(case, IRRELEVANT_DIR))

    def test_self_indirection(self):
        case = {
            "consumer": "${self:custom.forward} ${self:custom.four}",
            "custom": {
                "four": "4",
                "two": "2",
                "one": "1",
                "forward": "${self:custom.one} ${self:custom.two} ${self:custom.three}",
                "three": "${self:custom.tres}",
                "tres": "3"
            }
        }
        expected = {
            "consumer": "1 2 3 4",
            "custom": {
                "four": "4",
                "two": "2",
                "one": "1",
                "forward": "1 2 3",
                "three": "3",
                "tres": "3"
            }
        }
        self.assertEqual(expected, process_variables(case, IRRELEVANT_DIR))

    def test_self_circular_ref(self):
        case = {
            "custom": {
                "thing1": "${self:custom.thing2}",
                "thing2": "${self:custom.thing1}"
            }
        }
        result = process_variables(case, IRRELEVANT_DIR)
        # Undefined which will be picked, but won't be fully resolved
        self.assertTrue(result["custom"]["thing1"].startswith("${self:custom.thing"))
        self.assertTrue(result["custom"]["thing2"].startswith("${self:custom.thing"))

    def test_self_reference(self):
        case = {
            "me": "${self:me}"
        }
        expected = {
            "me": "${self:me}"
        }
        # Should just finish, no hang, no changes
        self.assertEqual(expected, process_variables(case, IRRELEVANT_DIR))

    def test_nested(self):
        case = {
            "second-value": "final",
            "first-value": "second",
            "consumer": "${self:${self:first-value}-value}"
        }
        expected = {
            "second-value": "final",
            "first-value": "second",
            "consumer": "final"
        }
        self.assertEqual(expected, process_variables(case, IRRELEVANT_DIR))

    def test_overwriting_variables(self):
        case = {
            "fallback": "final",
            "value": "${self:doesnt-exist, self:fallback}"
        }
        expected = {
            "fallback": "final",
            "value": "final"
        }
        self.assertEqual(expected, process_variables(case, IRRELEVANT_DIR))

    def test_custom_variable_syntax(self):
        case = {
            "provider": {
                "variableSyntax": "\\${{([ ~:a-zA-Z0-9._@\\'\",\\-\\/\\(\\)]+?)}}"
            },
            "custom": {
                "consumer": "${{self: custom.source}}",
                "source": "final"
            }
        }
        expected = {
            # NOTE: variableSyntax entry is removed to prevent self-matching
            "provider": {},
            "custom": {
                "consumer": "final",
                "source": "final"
            }
        }
        self.assertEqual(expected, process_variables(case, IRRELEVANT_DIR))

    def test_deep_var_override(self):
        case = {
            "custom": {
                "val1": "${self:not.a.value, 'bar'}",
                "val2": "${self:custom.val1}"
            }
        }
        expected = {
            "custom": {
                "val1": "bar",
                "val2": "bar"
            }
        }
        self.assertEqual(expected, process_variables(case, IRRELEVANT_DIR))

    def test_deep_references_into_deep_vars(self):
        case = {
            "custom": {
                "val0": {
                    "foo": "bar"
                },
                "val1": '${self:custom.val0}',
                "val2": '${self:custom.val1.foo}',
            }
        }
        expected = {
            "custom": {
                "val0": {
                    "foo": 'bar',
                },
                "val1": {
                    "foo": 'bar',
                },
                "val2": 'bar',
            }
        }
        self.assertEqual(expected, process_variables(case, IRRELEVANT_DIR))

    def test_quoted_vars(self):
        case = {
            "consumer": "${self: custom.bogus, \"value, with, commas\"}",
        }
        expected = {
            "consumer": "value, with, commas"
        }
        self.assertEqual(expected, process_variables(case, IRRELEVANT_DIR))

    def test_tokenize_by_commas(self):
        self.assertEqual(["single"],
                         _tokenize_by_commas("single"))
        self.assertEqual(["one", "two"],
                         _tokenize_by_commas("one,two"))
        self.assertEqual(["one", "two"],
                         _tokenize_by_commas("one, two"))
        # Double quotes
        self.assertEqual(["separate", "commas, in, value", "another"],
                         _tokenize_by_commas("separate, \"commas, in, value\", another"))
        self.assertEqual(["commas, in, value", "another"],
                         _tokenize_by_commas("\"commas, in, value\", another"))
        self.assertEqual(["separate", "commas, in, value"],
                         _tokenize_by_commas("separate, \"commas, in, value\""))
        # Single quotes
        self.assertEqual(["separate", "commas, in, value", "another"],
                         _tokenize_by_commas("separate, 'commas, in, value', another"))
        self.assertEqual(["commas, in, value", "another"],
                         _tokenize_by_commas("'commas, in, value', another"))
        self.assertEqual(["separate", "commas, in, value"],
                         _tokenize_by_commas("separate, 'commas, in, value'"))

    def test_token_to_type_and_loc(self):
        self.assertEqual(("self", "foo"),
                         _token_to_type_and_loc("self:foo"))
        self.assertEqual(("self", "foo"),
                         _token_to_type_and_loc("self: foo"))
        self.assertEqual(("something_made_up", "bar"),
                         _token_to_type_and_loc("something_made_up:bar"))
        self.assertEqual(("file(~/settings.yaml)", "foo"),
                         _token_to_type_and_loc("file(~/settings.yaml):foo"))
        self.assertEqual(("file(~/settings.yaml)", None),
                         _token_to_type_and_loc("file(~/settings.yaml)"))

    def test_parse_var(self):
        self.assertEqual(("self", "foo", "self", "bar"),
                         _parse_var("self:foo,self:bar"))
        self.assertEqual(("self", "foo", None, "bar"),
                         _parse_var("self:foo,bar"))
        self.assertEqual(("file(settings.yaml)", "foo", "self", "bar"),
                         _parse_var("file(settings.yaml):foo,self:bar"))
        self.assertEqual(("file(settings.yaml)", "foo", None, None),
                         _parse_var("file(settings.yaml):foo"))
        self.assertEqual(("file(settings.yaml)", None, None, None),
                         _parse_var("file(settings.yaml)"))

        self.assertEqual(("self", "foo", None, None),
                         _parse_var("self: foo"))       # eat whitespace

    #########################################
    # env/file variable resolution opt-out
    #
    # Default behavior (flag unset) resolves ${env:VAR} and ${file(path)},
    # preserving backward compatibility. That default path is already exercised
    # by the check tests (test_AdminPolicyDocument / test_S3PublicACLRead, which
    # rely on ${env:} and ${file()} resolution) and by the ${self:} tests above,
    # so it is not re-tested here. These tests cover only the NEW opt-out
    # behavior: CHECKOV_SERVERLESS_DISABLE_VARS=true disables ${env:} and
    # ${file()} resolution while leaving ${self:} resolution intact.

    @mock.patch.dict(os.environ, {"MY_SECRET": "s3cret", "CHECKOV_SERVERLESS_DISABLE_VARS": "true"})
    def test_env_var_not_resolved_when_disabled(self):
        """${env:VAR} must NOT resolve when CHECKOV_SERVERLESS_DISABLE_VARS=true."""
        case = {
            "provider": {"name": "aws"},
            "value": "${env:MY_SECRET}"
        }
        result = process_variables(case, IRRELEVANT_DIR)
        # Should remain unresolved
        self.assertEqual(result["value"], "${env:MY_SECRET}")

    @mock.patch.dict(os.environ, {"CHECKOV_SERVERLESS_DISABLE_VARS": "true"})
    def test_file_var_not_resolved_when_disabled(self):
        """${file(path)} must NOT resolve when CHECKOV_SERVERLESS_DISABLE_VARS=true."""
        with tempfile.NamedTemporaryFile(mode="w", suffix=".json", delete=False) as f:
            f.write('{"secret_key": "top-secret"}')
            tmp_path = f.name
        try:
            case = {
                "provider": {"name": "aws"},
                "value": "${file(" + tmp_path + "):secret_key}"
            }
            result = process_variables(case, IRRELEVANT_DIR)
            # Should remain unresolved
            self.assertEqual(result["value"], "${file(" + tmp_path + "):secret_key}")
        finally:
            os.unlink(tmp_path)

    @mock.patch.dict(os.environ, {"CHECKOV_SERVERLESS_DISABLE_VARS": "true"})
    def test_self_resolution_unaffected_when_disabled(self):
        """${self:} must always resolve even when CHECKOV_SERVERLESS_DISABLE_VARS=true."""
        case = {
            "provider": {"name": "aws"},
            "source": "resolved-value",
            "consumer": "${self:source}"
        }
        result = process_variables(case, IRRELEVANT_DIR)
        self.assertEqual(result["consumer"], "resolved-value")


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.SQLServerPublicAccessDisabled import check
from checkov.common.models.enums import CheckResult


class TestSQLServerPublicAccessDisabled(unittest.TestCase):

    def test_failure_1(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_mssql_server" "example" {
              name                         = "mssqlserver"
              resource_group_name          = azurerm_resource_group.example.name
              location                     = azurerm_resource_group.example.location
              version                      = "12.0"
              administrator_login          = "missadministrator"
              administrator_login_password = "thisIsKat11"
              minimum_tls_version          = "1.2"
              public_network_access_enabled = true
              azuread_administrator {
                login_username = "AzureAD Admin"
                object_id      = "00000000-0000-0000-0000-000000000000"
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_mssql_server']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_2(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_mssql_server" "example" {
              name                         = "mssqlserver"
              resource_group_name          = azurerm_resource_group.example.name
              location                     = azurerm_resource_group.example.location
              version                      = "12.0"
              administrator_login          = "missadministrator"
              administrator_login_password = "thisIsKat11"
              minimum_tls_version          = "1.2"
              azuread_administrator {
                login_username = "AzureAD Admin"
                object_id      = "00000000-0000-0000-0000-000000000000"
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_mssql_server']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_mssql_server" "example" {
              name                         = "mssqlserver"
              resource_group_name          = azurerm_resource_group.example.name
              location                     = azurerm_resource_group.example.location
              version                      = "12.0"
              administrator_login          = "missadministrator"
              administrator_login_password = "thisIsKat11"
              minimum_tls_version          = "1.2"
              public_network_access_enabled = false
              azuread_administrator {
                login_username = "AzureAD Admin"
                object_id      = "00000000-0000-0000-0000-000000000000"
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_mssql_server']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest

from checkov.terraform.checks.resource.gcp.GKEHasLabels import check
from checkov.common.models.enums import CheckResult


class TestGKEHasLabels(unittest.TestCase):

    def test_failure(self):
        resource_conf = {'name': ['google_cluster'], 'enable_legacy_abac': [False]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure2(self):
        resource_conf = {'name': ['google_cluster_bad'], 'monitoring_service': ['none'], 'enable_legacy_abac': [True], 'master_authorized_networks_config': [{'cidr_blocks': [{'cidr_block': ['0.0.0.0/0'], 'display_name': ['The world']}]}], 'master_auth': [{'username': ['test'], 'password': ['password']}], 'resource_labels': [{}]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {'name': ['google_cluster'], 'enable_legacy_abac': [False], 'resource_labels': [{'Owner': ['SomeoneNotWorkingHere']}]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import pytest

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.terraform.checks.resource.registry import resource_registry as registry

class TestStaticCheck(BaseResourceCheck):
    # for pytest not to collect this class as tests
    __test__ = False

    def __init__(self):
        name = "Test something"
        id = "CKV_TEST_1"
        supported_resources = ["ckv_test"]
        categories = [CheckCategories.CONVENTION]
        guideline = "https://docs.prismacloud.io/policy-reference/test-policies/test-resource-check"
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            guideline=guideline
        )

    def scan_resource_conf(self, conf):
        if "check_result" in conf.keys():
            check_result = conf["check_result"][0]
            if check_result:
                return CheckResult.PASSED

            return CheckResult.FAILED

        return CheckResult.UNKNOWN

@pytest.fixture(scope="module", autouse=True)
def remove_check():
    yield
    del registry.checks["ckv_test"]


@pytest.mark.parametrize(
    "conf,expected",
    [
        ({"check_result": [True]}, CheckResult.PASSED),
        ({"check_result": [False]}, CheckResult.FAILED),
        ({"foo": ["bar"]}, CheckResult.UNKNOWN),
        ({"count": [0], "check_result": [True]}, CheckResult.UNKNOWN),
        ({"count": [1], "check_result": [True]}, CheckResult.PASSED),
    ],
    ids=["pass", "fail", "unknown", "count_zero", "count_one"],
)
def test_scan_entity_conf(conf, expected):
    result = TestStaticCheck().scan_entity_conf(conf, "ckv_test")

    assert result == expected



import unittest

from checkov.common.models.enums import CheckResult
from checkov.common.output.secrets_record import SecretsRecord
from checkov.secrets.runner import Runner, ENTROPY_CHECK_IDS, GENERIC_PRIVATE_KEY_CHECK_IDS


class TestPrioritiseSecrets(unittest.TestCase):
    def setUp(self):
        self.secret_records = {
            'key1': SecretsRecord(check_id='CKV_SECRET_6', check_name='foo',
                                  check_result={"result": CheckResult.FAILED}, code_block=[(1, 'baz')],
                                  file_path='qux', file_line_range=[1, 2], resource='resource', evaluations=None,
                                  check_class='CheckClass', file_abs_path='abs_path'),
            'key2': SecretsRecord(check_id='CKV_SECRET_10', check_name='foo',
                                  check_result={"result": CheckResult.FAILED},
                                  code_block=[(1, 'baz')], file_path='qux', file_line_range=[1, 2], resource='resource',
                                  evaluations=None, check_class='CheckClass', file_abs_path='abs_path'),
            'key3': SecretsRecord(check_id='CKV_SECRET_18', check_name='foo',
                                  check_result={"result": CheckResult.FAILED}, code_block=[(1, 'baz')],
                                  file_path='qux', file_line_range=[1, 2], resource='resource', evaluations=None,
                                  check_class='CheckClass', file_abs_path='abs_path'),
            'key4': SecretsRecord(check_id='CKV_SECRET_192', check_name='foo',
                                  check_result={"result": CheckResult.FAILED}, code_block=[(1, 'baz')],
                                  file_path='qux', file_line_range=[1, 2], resource='resource', evaluations=None,
                                  check_class='CheckClass', file_abs_path='abs_path'),
        }
        self.ENTROPY_CHECK_IDS = ENTROPY_CHECK_IDS
        self.GENERIC_PRIVATE_KEY_CHECK_IDS = GENERIC_PRIVATE_KEY_CHECK_IDS

    def test_entropy_check_id_removed(self):
        result = Runner._prioritise_secrets(self.secret_records, 'key1', 'CKV_SECRET_18')
        self.assertTrue(result)
        self.assertNotIn('key1', self.secret_records)

    def test_generic_private_key_check_id_removed(self):
        result = Runner._prioritise_secrets(self.secret_records, 'key2', 'CKV_SECRET_18')
        self.assertTrue(result)
        self.assertNotIn('key2', self.secret_records)

    def test_generic_private_key_check_id_192_removed(self):
        result = Runner._prioritise_secrets(self.secret_records, 'key4', 'CKV_SECRET_18')
        self.assertTrue(result)
        self.assertNotIn('key4', self.secret_records)

    def test_no_removal_entropy_check_id(self):
        result = Runner._prioritise_secrets(self.secret_records, 'key1', 'CKV_SECRET_6')
        self.assertFalse(result)
        self.assertIn('key1', self.secret_records)

    def test_no_removal_generic_private_key_check_id(self):
        result = Runner._prioritise_secrets(self.secret_records, 'key2', 'CKV_SECRET_10')
        self.assertFalse(result)
        self.assertIn('key2', self.secret_records)

    def test_no_removal_generic_private_key_check_id_192(self):
        result = Runner._prioritise_secrets(self.secret_records, 'key4', 'CKV_SECRET_192')
        self.assertFalse(result)
        self.assertIn('key4', self.secret_records)

    def test_no_removal_other_check_id(self):
        result = Runner._prioritise_secrets(self.secret_records, 'key3', 'CKV_SECRET_1000')
        self.assertFalse(result)
        self.assertIn('key3', self.secret_records)

    def test_no_removal_of_first_check_id(self):
        result = Runner._prioritise_secrets(self.secret_records, 'key1', 'CKV_SECRET_80')
        self.assertFalse(result)
        self.assertIn('key1', self.secret_records)


if __name__ == '__main__':
    unittest.main()



"""Chokepoint tests for ``Checkov.get_external_checks_dir``.

Bypasses ``Checkov.__init__`` (which runs heavy ``parse_config`` setup)
by constructing instances via ``__new__`` and stubbing ``config`` and
``exit_run`` to the minimum the method needs. This keeps the test
focused on the verification chokepoint behaviour:

* No key configured → today's behaviour (no verification, no registry).
* Key configured + signed dir → verification succeeds, registry active.
* Key configured + unsigned/modified dir → exit_run() invoked, no scan.
* Platform-supplied ``sast_custom_policies`` is appended **after**
  verification and is not sent through it.
"""
from __future__ import annotations

import types
from pathlib import Path
from unittest.mock import patch

import pytest

from checkov.common.external_checks.verification.sources_registry import (
    is_verification_active,
    reset_for_tests,
)


@pytest.fixture(autouse=True)
def _reset_registry():
    reset_for_tests()
    try:
        yield
    finally:
        reset_for_tests()


def test_no_keys_no_verification(valid_dir: Path, make_checkov):
    """No --external-checks-public-key → registry stays inactive."""
    checkov = make_checkov(external_checks_dir=[str(valid_dir)])

    # Patch bc_integration.sast_custom_policies to None for isolation.
    with patch("checkov.main.bc_integration") as bc:
        bc.sast_custom_policies = None
        dirs = checkov.get_external_checks_dir()

    assert dirs == [str(valid_dir)]
    assert is_verification_active() is False


def test_no_external_checks_dir_skips_verification(
    key_a_pub_pem: bytes, tmp_path: Path, make_checkov,
):
    """Key configured but no dirs to verify → no-op, no error."""
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)

    checkov = make_checkov(
        external_checks_dir=None,
        external_checks_public_key=[str(key_path)],
    )
    with patch("checkov.main.bc_integration") as bc:
        bc.sast_custom_policies = None
        dirs = checkov.get_external_checks_dir()

    # No user-supplied dirs and no platform dir → empty list-like (None or []).
    assert not dirs
    assert is_verification_active() is False


def test_keys_plus_signed_dir_activates_registry(
    valid_dir: Path, key_a_pub_pem: bytes, tmp_path: Path, make_checkov,
):
    """Happy path: signed dir + matching key → registry populated, dir returned."""
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)

    checkov = make_checkov(
        external_checks_dir=[str(valid_dir)],
        external_checks_public_key=[str(key_path)],
    )
    with patch("checkov.main.bc_integration") as bc:
        bc.sast_custom_policies = None
        dirs = checkov.get_external_checks_dir()

    assert dirs == [str(valid_dir)]
    assert is_verification_active() is True


def test_keys_plus_unsigned_dir_exits_via_exit_run(
    unsigned_dir: Path, key_a_pub_pem: bytes, tmp_path: Path, make_checkov,
):
    """Verification failure routes the exit through ``self.exit_run()``.

    ``exit_run()`` honours ``--no-fail-on-crash`` — the documented contract
    of that flag is to "Return exit code 0 instead of 2". The chokepoint
    therefore reports the failure (stderr + log file), then delegates the
    actual ``exit()`` call to ``self.exit_run()`` so the flag's contract
    is preserved end-to-end. Verified here by stubbing ``exit_run`` to a
    sentinel and asserting it fires.
    """
    import os
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)

    checkov = make_checkov(
        external_checks_dir=[str(unsigned_dir)],
        external_checks_public_key=[str(key_path)],
    )

    exit_run_called: list[bool] = []

    def _fake_exit_run(self):
        exit_run_called.append(True)
        # Mirror the real exit_run's terminating behaviour so the caller
        # doesn't accidentally proceed past the failure point.
        raise SystemExit(2 if not self.config.no_fail_on_crash else 0)

    checkov.exit_run = types.MethodType(_fake_exit_run, checkov)  # type: ignore[attr-defined]

    prev_cwd = os.getcwd()
    os.chdir(str(tmp_path))
    try:
        with patch("checkov.main.bc_integration") as bc:
            bc.sast_custom_policies = None
            with pytest.raises(SystemExit) as raised:
                checkov.get_external_checks_dir()
    finally:
        os.chdir(prev_cwd)

    assert exit_run_called, (
        "verification failure must delegate exit to self.exit_run() so the "
        "--no-fail-on-crash contract is preserved; got direct sys.exit instead"
    )
    assert raised.value.code == 2, (
        f"with --no-fail-on-crash unset, exit must be 2; got {raised.value.code!r}"
    )
    assert is_verification_active() is False


def test_platform_policies_appended_after_verification(
    valid_dir: Path, key_a_pub_pem: bytes, tmp_path: Path, make_checkov,
):
    """sast_custom_policies dir is appended after verification.

    The platform-supplied dir must NOT pass through verify_and_register
    — it reaches Checkov through a different trust boundary (the
    integration's TLS-authenticated fetch) and is not part of the
    trailer-signing scope.
    """
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)

    # Make a platform-style dir that would fail verification if mistakenly
    # included — contains an unsigned .py file.
    platform_dir = tmp_path / "platform_policies"
    platform_dir.mkdir()
    (platform_dir / "unsigned_platform.py").write_bytes(b"# unsigned\n")

    checkov = make_checkov(
        external_checks_dir=[str(valid_dir)],
        external_checks_public_key=[str(key_path)],
    )

    with patch("checkov.main.bc_integration") as bc:
        bc.sast_custom_policies = str(platform_dir)
        dirs = checkov.get_external_checks_dir()

    assert str(valid_dir) in dirs
    assert str(platform_dir) in dirs
    assert dirs.index(str(valid_dir)) < dirs.index(str(platform_dir))
    # And critically — verification did NOT raise on the platform dir.
    assert is_verification_active() is True


def test_cli_flag_is_registered_in_parser():
    """Smoke test: --external-checks-public-key parses with default None."""
    from checkov.common.util.ext_argument_parser import ExtArgumentParser

    parser = ExtArgumentParser(description="test")
    parser.add_parser_args()
    cfg = parser.parse_args([])
    assert cfg.external_checks_public_key is None

    cfg2 = parser.parse_args(
        ["--external-checks-public-key", "/tmp/a.pem",
         "--external-checks-public-key", "/tmp/b.pem"]
    )
    assert cfg2.external_checks_public_key == ["/tmp/a.pem", "/tmp/b.pem"]


def test_help_text_documents_no_fail_on_crash_override():
    """``--external-checks-public-key`` help mentions the ``--no-fail-on-crash`` override.

    Pins S5: the help text must accurately describe the runtime
    behaviour. The flag changes the exit code from 2 to 0 on
    verification failure (the documented contract of
    ``--no-fail-on-crash`` is "Return exit code 0 instead of 2", and
    verification failures honour that contract — see
    ``test_verification_failure_with_no_fail_on_crash_exits_0`` in
    ``test_regression.py``). The help text must mention this so
    operators see one source of truth, not a docs-vs-runtime drift.
    """
    from checkov.common.util.ext_argument_parser import ExtArgumentParser

    parser = ExtArgumentParser(description="test")
    parser.add_parser_args()

    action = next(
        (a for a in parser._actions if "--external-checks-public-key" in (a.option_strings or [])),
        None,
    )
    assert action is not None, (
        "could not find --external-checks-public-key action on the parser"
    )
    help_text = action.help or ""

    # The baseline exit code is still 2 — the help must keep saying so.
    assert "exit with code 2" in help_text, (
        f"help text must still mention the default 'exit with code 2'; "
        f"got: {help_text!r}"
    )
    # AND it must mention the --no-fail-on-crash override.
    assert "--no-fail-on-crash" in help_text, (
        f"help text must mention the --no-fail-on-crash override so "
        f"operators see one source of truth; got: {help_text!r}"
    )



from pathlib import Path

from checkov.arm.utils import get_files_definitions, extract_resource_name_from_reference_func


def test_get_files_definitions_with_parsing_error():
    # given
    file_path = Path(__file__).parent / "parser/examples/json/with_comments.json"

    # when
    definitions, definitions_raw, parsing_errors = get_files_definitions([str(file_path)])

    # then
    assert definitions == {}
    assert definitions_raw == {}
    assert len(parsing_errors) == 1
    assert parsing_errors[0].endswith("parser/examples/json/with_comments.json")


def test_extract_resource_name_from_reference_func():
    test_cases = ["reference('storageAccountName')",
                  "reference('myStorage').primaryEndpoints",
                  "reference('myStorage', '2022-09-01', 'Full').location",
                  "reference(resourceId('storageResourceGroup', 'Microsoft.Storage/storageAccounts', "
                  "'storageAccountName')), '2022-09-01')",
                  "reference(resourceId('Microsoft.Network/publicIPAddresses', 'ipAddressName'))"]

    expected = ["storageAccountName", "myStorage", "myStorage", "storageAccountName", "ipAddressName"]

    for i, test_case in enumerate(test_cases):
        assert extract_resource_name_from_reference_func(test_case) == expected[i]



import os

from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.graph_builder.graph_components.block_types import BlockType
from checkov.kubernetes.graph_manager import KubernetesGraphManager
from checkov.kubernetes.parser.parser import parse
from tests.kubernetes.graph.base_graph_tests import TestGraph
from checkov.kubernetes.kubernetes_graph_flags import K8sGraphFlags

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


class TestKubernetesGraphManager(TestGraph):
    def test_build_graph_from_source_directory_no_rendering(self):
        root_dir = os.path.realpath(os.path.join(TEST_DIRNAME, "../runner/resources"))
        graph_manager = KubernetesGraphManager(db_connector=NetworkxConnector())
        graph_flags = K8sGraphFlags(create_complex_vertices=False, create_edges=False)
        graph_manager.graph_flags = graph_flags
        local_graph, definitions = graph_manager.build_graph_from_source_directory(root_dir, render_variables=False)

        expected_resources_by_file = {
            os.path.join(root_dir, "example.yaml"): [
                "Service.default.kafka-hs"],
            os.path.join(root_dir, "example_multiple.yaml"): [
                "PodDisruptionBudget.a.a",
                "Service.default.a"],
            os.path.join(root_dir, "graph.yaml"): [
                "StatefulSet.default.cassandra",
                "Deployment.default.my-nginx",
                "Pod.default.cassandra.app-cassandra",
                "Pod.default.my-nginx.app-nginx"]
        }
        self.assertEqual(7, len(local_graph.vertices))
        self.assertEqual(7, len(local_graph.vertices_by_block_type[BlockType.RESOURCE]))

        for v in local_graph.vertices:
            self.assertIn(v.name, expected_resources_by_file[v.path])

    def test_build_graph_from_definitions(self):
        relative_file_path = "../checks/example_AllowedCapabilities/cronjob-PASSED.yaml"
        definitions = {}
        graph_flags = K8sGraphFlags(create_complex_vertices=False, create_edges=False)
        file = os.path.realpath(os.path.join(TEST_DIRNAME, relative_file_path))
        (definitions[relative_file_path], definitions_raw) = parse(file)
        resource = definitions[relative_file_path][0]

        graph_manager = KubernetesGraphManager(db_connector=NetworkxConnector())
        graph_manager.graph_flags = graph_flags
        local_graph = graph_manager.build_graph_from_definitions(definitions)
        self.assertEqual(1, len(local_graph.vertices))
        self.assert_vertex(local_graph.vertices[0], resource)



from aws_cdk import core
from aws_cdk import aws_neptune as neptune

class MyNeptuneStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create a Neptune DB cluster with storage encryption enabled
        neptune_cluster = neptune.CfnDBCluster(
            self,
            "MyNeptuneCluster",
            engine="neptune",
            db_cluster_identifier="my-neptune-cluster",
            master_username="admin",
            master_user_password="mypassword", # checkov:skip=CKV_SECRET_6 test secret
            storage_encrypted=False,  # Enable storage encryption
            port=8182,  # Specify the port as needed
            availability_zones=["us-east-1a", "us-east-1b"],  # Specify the availability zones
        )

class MyNeptuneStack2(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create a Neptune DB cluster with storage encryption enabled
        neptune_cluster = neptune.DatabaseCluster(
            self,
            "MyNeptuneCluster",
            engine=neptune.DatabaseClusterEngine.NEPTUNE,
            master_user=neptune.Login(
                username="admin",
                password="mypassword", # checkov:skip=CKV_SECRET_6 test secret
            ),
            default_database_name="mydb",
            removal_policy=core.RemovalPolicy.DESTROY,  # Set the removal policy as needed
            vpc=your_vpc,  # Specify the VPC where the cluster should be deployed
            instances=1,  # Specify the number of instances
        )




from collections import Counter
from pathlib import Path

from checkov.bicep.graph_builder.graph_components.block_types import BlockType
from checkov.bicep.graph_builder.local_graph import BicepLocalGraph
from checkov.bicep.parser import Parser

EXAMPLES_DIR = Path(__file__).parent.parent.parent / "examples"


def test_build_graph():
    # given
    test_file = EXAMPLES_DIR / "playground.bicep"
    template, _ = Parser().parse(test_file)
    local_graph = BicepLocalGraph(definitions={test_file: template})

    # when
    local_graph.build_graph(render_variables=False)

    # then
    assert len(local_graph.vertices) == 24
    assert len(local_graph.edges) == 33

    assert len(local_graph.vertices_by_block_type[BlockType.TARGET_SCOPE]) == 1
    assert len(local_graph.vertices_by_block_type[BlockType.PARAM]) == 5
    assert len(local_graph.vertices_by_block_type[BlockType.VAR]) == 10
    assert len(local_graph.vertices_by_block_type[BlockType.RESOURCE]) == 8
    assert len(local_graph.vertices_by_block_type[BlockType.MODULE]) == 0
    assert len(local_graph.vertices_by_block_type[BlockType.OUTPUT]) == 0

    out_edge_counts = Counter([e.origin for e in local_graph.edges])
    in_edge_counts = Counter([e.dest for e in local_graph.edges])

    assert out_edge_counts == Counter(
        {
            16: 9,
            19: 5,
            20: 6,
            17: 3,
            18: 2,
            21: 4,
            22: 2,
            23: 2,
        }
    )
    assert in_edge_counts == Counter(
        {
            5: 8,
            6: 2,
            2: 1,
            3: 1,
            1: 1,
            20: 1,
            21: 1,
            17: 1,
            13: 1,
            4: 1,
            15: 1,
            9: 1,
            10: 2,
            11: 2,
            18: 1,
            7: 1,
            22: 1,
            23: 1,
            8: 1,
            12: 1,
            14: 1,
            19: 2,
        }
    )



from __future__ import annotations

import logging
import os
from pathlib import Path
from typing import TYPE_CHECKING, Any
from typing_extensions import TypeAlias  # noqa[TC002]

from checkov.cloudformation import cfn_utils
from checkov.cloudformation.context_parser import ContextParser as CfnContextParser
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.util.secrets import omit_secret_value_from_checks
from checkov.serverless.base_registry import EntityDetails
from checkov.serverless.graph_builder.definition_context import build_definitions_context
from checkov.serverless.graph_builder.graph_to_definitions import convert_graph_vertices_to_definitions
from checkov.serverless.graph_builder.local_graph import ServerlessLocalGraph
from checkov.serverless.graph_manager import ServerlessGraphManager
from checkov.serverless.parsers.context_parser import ContextParser as SlsContextParser, ContextParser
from checkov.cloudformation.checks.resource.registry import cfn_registry
from checkov.serverless.checks.complete.registry import complete_registry
from checkov.serverless.checks.custom.registry import custom_registry
from checkov.serverless.checks.function.registry import function_registry
from checkov.serverless.checks.layer.registry import layer_registry
from checkov.serverless.checks.package.registry import package_registry
from checkov.serverless.checks.plugin.registry import plugin_registry
from checkov.serverless.checks.provider.registry import provider_registry
from checkov.serverless.checks.service.registry import service_registry
from checkov.common.runners.base_runner import BaseRunner
from checkov.runner_filter import RunnerFilter
from checkov.common.checks_infra.registry import get_graph_checks_registry
from checkov.common.output.record import Record
from checkov.common.output.report import Report
from checkov.common.graph.graph_builder.consts import GraphSource
from checkov.common.output.extra_resource import ExtraResource
from checkov.serverless.parsers.parser import CFN_RESOURCES_TOKEN
from checkov.serverless.utils import get_scannable_file_paths, get_files_definitions, SLS_FILE_MASK, get_resource_tags

if TYPE_CHECKING:
    from checkov.common.graph.checks_infra.registry import BaseRegistry
    from checkov.common.typing import LibraryGraphConnector

MULTI_ITEM_SECTIONS = [
    ("functions", function_registry),
    ("layers", layer_registry)
]
SINGLE_ITEM_SECTIONS = [
    ("provider", provider_registry),
    ("custom", custom_registry),
    ("package", package_registry),
    ("plugins", plugin_registry),
    ("service", service_registry)
]

_ServerlessContext: TypeAlias = "dict[str, dict[str, Any]]"
_ServerlessDefinitions: TypeAlias = "dict[str, dict[str, Any]]"


class Runner(BaseRunner[_ServerlessDefinitions, _ServerlessContext, ServerlessGraphManager]):
    check_type = CheckType.SERVERLESS  # noqa: CCE003  # a static attribute

    def __init__(self,
                 db_connector: LibraryGraphConnector | None = None,
                 source: str = GraphSource.SERVERLESS,
                 graph_class: type[ServerlessLocalGraph] = ServerlessLocalGraph,
                 graph_manager: ServerlessGraphManager | None = None,
                 external_registries: list[BaseRegistry] | None = None,
                 ) -> None:
        super().__init__(file_names=SLS_FILE_MASK)

        db_connector = db_connector or self.db_connector
        self.external_registries = external_registries if external_registries else []
        self.graph_class = graph_class
        self.graph_manager: "ServerlessGraphManager" = (
            graph_manager if graph_manager else ServerlessGraphManager(source=source, db_connector=db_connector)
        )
        self.graph_registry = get_graph_checks_registry(self.check_type)

        self.definitions: _ServerlessDefinitions = {}
        self.definitions_raw: dict[str, list[tuple[int, str]]] = {}
        self.context: _ServerlessContext | None = None
        self.root_folder: "str | None" = None

    def run(
            self,
            root_folder: str | None,
            external_checks_dir: list[str] | None = None,
            files: list[str] | None = None,
            runner_filter: RunnerFilter | None = None,
            collect_skip_comments: bool = True,
    ) -> Report:
        runner_filter = runner_filter or RunnerFilter()
        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        report = Report(self.check_type)
        self.root_folder = root_folder

        if not self.context or not self.definitions:
            files_list: list[str] = []
            filepath_fn = None
            if external_checks_dir:
                for directory in external_checks_dir:
                    function_registry.load_external_checks(directory)

            if files:
                files_list = [file for file in files if os.path.basename(file) in SLS_FILE_MASK]

            if self.root_folder:
                files_list = get_scannable_file_paths(self.root_folder, runner_filter.excluded_paths)

            definitions, definitions_raw = get_files_definitions(files_list, filepath_fn)

            # Filter out empty files that have not been parsed successfully
            self.definitions = {k: v for k, v in definitions.items() if v}
            self.definitions_raw = {k: v for k, v in definitions_raw.items() if k in definitions.keys()}
            self.context = build_definitions_context(definitions=self.definitions, definitions_raw=self.definitions_raw)

            logging.info("Creating Serverless graph")
            local_graph = self.graph_manager.build_graph_from_definitions(definitions=self.definitions)
            logging.info(f'Successfully created Serverless graph ({len(local_graph.vertices)} vertices)')

            self.graph_manager.save_graph(local_graph)
            self.definitions, self.breadcrumbs = convert_graph_vertices_to_definitions(
                vertices=local_graph.vertices,
                root_folder=root_folder,
            )

        self.pbar.initiate(len(self.definitions))

        self.add_python_check_results(report, runner_filter)

        return report

    def add_python_check_results(self, report: Report, runner_filter: RunnerFilter) -> None:
        for sls_file, sls_file_data in self.definitions.items():
            self.pbar.set_additional_data({'Current File Scanned': os.path.relpath(sls_file, self.root_folder)})
            if not isinstance(sls_file_data, dict):
                continue

            sls_context_parser = SlsContextParser(sls_file, sls_file_data, self.definitions_raw[sls_file])

            self.cfn_resources_checks(sls_file, sls_file_data, report, runner_filter, sls_context_parser)
            self.multi_item_sections_checks(sls_file, sls_file_data, report, runner_filter, sls_context_parser)
            self.single_item_sections_checks(sls_file, sls_file_data, report, runner_filter, sls_context_parser)
            self.complete_python_checks(sls_file, sls_file_data, report, runner_filter, sls_context_parser)

    def complete_python_checks(self,
                               sls_file: str,
                               sls_file_data: dict[str, Any],
                               report: Report,
                               runner_filter: RunnerFilter,
                               sls_context_parser: ContextParser) -> None:
        # "Complete" checks
        # NOTE: Ignore code content, no point in showing (could be long)
        file_abs_path = Path(sls_file).absolute()
        entity_code_lines = self.definitions_raw[sls_file]
        entity_lines_range = [1, len(entity_code_lines) - 1]
        skipped_checks = CfnContextParser.collect_skip_comments(entity_code_lines or [])
        variable_evaluations: dict[str, Any] = {}
        entity = EntityDetails(sls_context_parser.provider_type, sls_file_data)
        results = complete_registry.scan(sls_file, entity, skipped_checks, runner_filter)
        tags = cfn_utils.get_resource_tags(entity, complete_registry)  # type:ignore[arg-type]
        if results:
            for check, check_result in results.items():
                record = Record(check_id=check.id, check_name=check.name, check_result=check_result,
                                code_block=[],  # Don't show, could be large
                                file_path=self.extract_file_path_from_abs_path(Path(sls_file)),
                                file_line_range=entity_lines_range,
                                resource="complete",  # Weird, not sure what to put where
                                evaluations=variable_evaluations,
                                check_class=check.__class__.__module__,
                                file_abs_path=str(file_abs_path),
                                entity_tags=tags, severity=check.severity)
                record.set_guideline(check.guideline)
                report.add_record(record=record)
        else:
            report.extra_resources.add(
                ExtraResource(
                    file_abs_path=str(file_abs_path),
                    file_path=self.extract_file_path_from_abs_path(Path(sls_file)),
                    resource="complete",
                )
            )

    def single_item_sections_checks(self,
                                    sls_file: str,
                                    sls_file_data: dict[str, Any],
                                    report: Report,
                                    runner_filter: RunnerFilter,
                                    sls_context_parser: ContextParser) -> None:
        # Sub-sections that are a single item
        file_abs_path = Path(sls_file).absolute()
        for token, registry in SINGLE_ITEM_SECTIONS:
            item_content = sls_file_data.get(token)
            if not item_content:
                continue
            entity_lines_range, entity_code_lines = sls_context_parser.extract_code_lines(item_content)
            if not entity_lines_range:
                entity_code_lines = self.definitions_raw[sls_file]
                entity_lines_range = [1, len(entity_code_lines) - 1]

            skipped_checks = CfnContextParser.collect_skip_comments(entity_code_lines or [])
            variable_evaluations: dict[str, Any] = {}
            entity = EntityDetails(sls_context_parser.provider_type, item_content)
            results = registry.scan(sls_file, entity, skipped_checks, runner_filter)
            tags = get_resource_tags(entity, registry)
            fname = Path(sls_context_parser.file(item_content)).resolve()

            if results:
                for check, check_result in results.items():
                    censored_code_lines = omit_secret_value_from_checks(
                        check=check,
                        check_result=check_result,
                        entity_code_lines=entity_code_lines or [],
                        entity_config=item_content,
                        resource_attributes_to_omit=runner_filter.resource_attr_to_omit
                    )
                    record = Record(
                        check_id=check.id,
                        check_name=check.name,
                        check_result=check_result,
                        code_block=censored_code_lines,
                        file_path=self.extract_file_path_from_abs_path(fname),
                        file_line_range=entity_lines_range or [0, 0],
                        resource=token,
                        evaluations=variable_evaluations,
                        check_class=check.__class__.__module__,
                        file_abs_path=str(file_abs_path),
                        entity_tags=tags,
                        severity=check.severity,
                    )
                    record.set_guideline(check.guideline)
                    report.add_record(record=record)
            else:
                report.extra_resources.add(
                    ExtraResource(
                        file_abs_path=str(file_abs_path),
                        file_path=self.extract_file_path_from_abs_path(Path(sls_file)),
                        resource=token,
                    )
                )

    def multi_item_sections_checks(self,
                                   sls_file: str,
                                   sls_file_data: dict[str, Any],
                                   report: Report,
                                   runner_filter: RunnerFilter,
                                   sls_context_parser: ContextParser) -> None:
        # Sub-sections that have multiple items under them
        file_abs_path = Path(sls_file).absolute()
        for token, registry in MULTI_ITEM_SECTIONS:
            template_items = sls_file_data.get(token)
            if not template_items or not isinstance(template_items, dict):
                continue
            for item_name, item_content in template_items.items():
                if not isinstance(item_content, dict):
                    continue
                entity_lines_range, entity_code_lines = sls_context_parser.extract_code_lines(item_content)
                if entity_lines_range and entity_code_lines:
                    skipped_checks = CfnContextParser.collect_skip_comments(entity_code_lines)
                    variable_evaluations: dict[str, Any] = {}
                    if token == "functions":  # nosec
                        # "Enriching" copies things like "environment" and "stackTags" down into the
                        # function data from the provider block since logically that's what serverless
                        # does. This allows checks to see what the complete data would be.
                        sls_context_parser.enrich_function_with_provider(item_name)
                    entity = EntityDetails(sls_context_parser.provider_type, item_content)
                    results = registry.scan(sls_file, entity, skipped_checks, runner_filter)
                    tags = get_resource_tags(entity, registry)
                    fname = Path(sls_context_parser.file(item_content)).resolve()
                    if results:
                        for check, check_result in results.items():
                            censored_code_lines = omit_secret_value_from_checks(
                                check=check,
                                check_result=check_result,
                                entity_code_lines=entity_code_lines,
                                entity_config=item_content,
                                resource_attributes_to_omit=runner_filter.resource_attr_to_omit
                            )
                            record = Record(check_id=check.id, check_name=check.name, check_result=check_result,
                                            code_block=censored_code_lines,
                                            file_path=self.extract_file_path_from_abs_path(fname),
                                            file_line_range=entity_lines_range,
                                            resource=item_name, evaluations=variable_evaluations,
                                            check_class=check.__class__.__module__,
                                            file_abs_path=str(file_abs_path),
                                            entity_tags=tags, severity=check.severity)
                            record.set_guideline(check.guideline)
                            report.add_record(record=record)
                    else:
                        report.extra_resources.add(
                            ExtraResource(
                                file_abs_path=str(file_abs_path),
                                file_path=self.extract_file_path_from_abs_path(Path(sls_file)),
                                resource=item_name,
                            )
                        )

    def cfn_resources_checks(self,
                             sls_file: str,
                             sls_file_data: dict[str, Any],
                             report: Report,
                             runner_filter: RunnerFilter,
                             sls_context_parser: SlsContextParser) -> None:
        file_abs_path = Path(sls_file).absolute()
        if CFN_RESOURCES_TOKEN in sls_file_data and isinstance(sls_file_data[CFN_RESOURCES_TOKEN], dict):
            cf_sub_template = sls_file_data[CFN_RESOURCES_TOKEN]
            cf_sub_resources = cf_sub_template.get("Resources")
            if cf_sub_resources and isinstance(cf_sub_resources, dict):
                cf_context_parser = CfnContextParser(sls_file, cf_sub_template, self.definitions_raw[sls_file])
                logging.debug(f"Template Dump for {sls_file}: {sls_file_data}")
                cf_context_parser.evaluate_default_refs()
                for resource_name, resource in cf_sub_resources.items():
                    if not isinstance(resource, dict):
                        continue
                    cf_resource_id = cf_context_parser.extract_cf_resource_id(resource, resource_name)
                    if not cf_resource_id:
                        # Not Type attribute for resource
                        continue
                    report.add_resource(f'{file_abs_path}:{cf_resource_id}')
                    entity_lines_range, entity_code_lines = sls_context_parser.extract_code_lines(resource)
                    if entity_lines_range and entity_code_lines:
                        skipped_checks = CfnContextParser.collect_skip_comments(entity_code_lines)
                        # TODO - Variable Eval Message!
                        variable_evaluations: dict[str, Any] = {}

                        entity_dict = {resource_name: resource}
                        results = cfn_registry.scan(sls_file, entity_dict, skipped_checks, runner_filter)
                        tags = cfn_utils.get_resource_tags(entity_dict, cfn_registry)
                        if results:
                            for check, check_result in results.items():
                                censored_code_lines = omit_secret_value_from_checks(
                                    check=check,
                                    check_result=check_result,
                                    entity_code_lines=entity_code_lines,
                                    entity_config=resource,
                                    resource_attributes_to_omit=runner_filter.resource_attr_to_omit
                                )
                                record = Record(check_id=check.id, bc_check_id=check.bc_id, check_name=check.name,
                                                check_result=check_result,
                                                code_block=censored_code_lines,
                                                file_path=self.extract_file_path_from_abs_path(Path(sls_file)),
                                                file_line_range=entity_lines_range,
                                                resource=cf_resource_id, evaluations=variable_evaluations,
                                                check_class=check.__class__.__module__,
                                                file_abs_path=str(file_abs_path),
                                                entity_tags=tags, severity=check.severity)
                                record.set_guideline(check.guideline)
                                report.add_record(record=record)
                        else:
                            report.extra_resources.add(
                                ExtraResource(
                                    file_abs_path=str(file_abs_path),
                                    file_path=self.extract_file_path_from_abs_path(Path(sls_file)),
                                    resource=cf_resource_id,
                                )
                            )

    def extract_file_path_from_abs_path(self, path: Path) -> str:
        return f"{os.path.sep}{os.path.relpath(path, self.root_folder)}"

    def set_definitions_raw(self, definitions_raw: dict[str, list[tuple[int, str]]]) -> None:
        self.definitions_raw = definitions_raw



import unittest
from pathlib import Path

from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.azure.FunctionAppMinTLSVersion import check
from checkov.terraform.runner import Runner


class TestFunctionAppMinTLSVersion(unittest.TestCase):
    def test(self):
        test_files_dir = Path(__file__).parent / "example_FunctionAppMinTLSVersion"

        report = Runner().run(root_folder=test_files_dir, runner_filter=RunnerFilter(checks=[check.id]))
        summary = report.get_summary()

        passing_resources = {
            "azurerm_function_app.pass",
            "azurerm_function_app.pass2",
            "azurerm_function_app.pass3",
            "azurerm_function_app_slot.pass4",
            "azurerm_linux_function_app.pass5",
            "azurerm_windows_function_app.pass6",
            "azurerm_linux_function_app_slot.pass7",
            "azurerm_windows_function_app_slot.pass8",
            "azurerm_windows_function_app_slot.pass9",
        }
        failing_resources = {
            "azurerm_function_app.fail",
            "azurerm_function_app_slot.fail2",
            "azurerm_linux_function_app.fail3",
            "azurerm_windows_function_app.fail4",
            "azurerm_linux_function_app_slot.fail5",
            "azurerm_windows_function_app_slot.fail6",
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(summary["passed"], len(passing_resources))
        self.assertEqual(summary["failed"], len(failing_resources))
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)


if __name__ == "__main__":
    unittest.main()



"""Test if OCI policy statements are evaluated correctly."""

from pathlib import Path
from unittest import mock

from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.terraform.graph_manager import TerraformGraphManager

def test_oci_policy_statements_with_provider_env_var():
    # given
    resources_dir = Path(__file__).parent.parent / "resources/oci_policies"
    graph_manager = TerraformGraphManager(db_connector=NetworkxConnector())

    # when
    local_graph, _ = graph_manager.build_graph_from_source_directory(
        source_dir=str(resources_dir), render_variables=True
    )

    # then
    statements = local_graph.vertices[0].config["oci_identity_policy"]["example"]["statements"][0]
    assert statements == [
        "allow group group-admin-001 to use groups in tenancy where target.group.name != 'Administrators'"
    ]



import unittest

from checkov.common.checks.base_check import BaseCheck
from checkov.common.checks.base_check_registry import BaseCheckRegistry


class TestCheck(BaseCheck):
    # for pytest not to collect this class as tests
    __test__ = False

    def __init__(self, *supported_entities, id="CKV_T_1"):
        name = "Example check"
        categories = []
        supported_entities = list(supported_entities)
        block_type = "module"
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_entities,
                         block_type=block_type)

    def scan_entity_conf(self, conf, entity_type):
        pass


# noinspection DuplicatedCode
class TestRunnerRegistry(unittest.TestCase):

    def test_add_non_wildcard(self):
        registry = BaseCheckRegistry('')
        resource_1_check = TestCheck("resource_1")
        registry.register(resource_1_check)
        checks = registry.get_checks("resource_1")
        self.assertEqual(1, len(checks))
        self.assertEqual(resource_1_check, checks[0])

        self.assertIn("resource_1", registry.checks)
        self.assertNotIn("resource_1", registry.wildcard_checks)

    def test_add_wildcard(self):
        registry = BaseCheckRegistry('')
        resource_s_check = TestCheck("resource_*")
        registry.register(resource_s_check)
        checks = registry.get_checks("resource_*")
        self.assertEqual(1, len(checks))
        self.assertEqual(resource_s_check, checks[0])

        self.assertNotIn("resource_*", registry.checks)
        self.assertIn("resource_*", registry.wildcard_checks)

    def test__is_wildcard(self):
        self.assertFalse(BaseCheckRegistry._is_wildcard(""))
        self.assertFalse(BaseCheckRegistry._is_wildcard("resource"))
        self.assertFalse(BaseCheckRegistry._is_wildcard("module"))
        self.assertFalse(BaseCheckRegistry._is_wildcard("aws_s3_bucket"))
        self.assertTrue(BaseCheckRegistry._is_wildcard("aws_*"))
        self.assertTrue(BaseCheckRegistry._is_wildcard("*"))
        self.assertTrue(BaseCheckRegistry._is_wildcard("aws_[^0-9]"))

    def test_get_check_by_id(self):
        registry = BaseCheckRegistry('')
        resource_1_check = TestCheck("resource_1", id="CKV_T_1")
        resource_2_check = TestCheck("resource_2", id="CKV_T_2")
        resource_as_check = TestCheck("resource_a*", id="CKV_T_3")
        resource_bs_check = TestCheck("resource_b*", id="CKV_T_4")
        registry.register(resource_1_check)
        registry.register(resource_2_check)
        registry.register(resource_as_check)
        registry.register(resource_bs_check)

        self.assertEqual(resource_1_check, registry.get_check_by_id("CKV_T_1"))
        self.assertEqual(resource_2_check, registry.get_check_by_id("CKV_T_2"))
        self.assertEqual(resource_as_check, registry.get_check_by_id("CKV_T_3"))
        self.assertEqual(resource_bs_check, registry.get_check_by_id("CKV_T_4"))
        self.assertIsNone(registry.get_check_by_id("CKV_T_5"))

    def test_get_check_no_wildcard(self):
        registry = BaseCheckRegistry('')
        resource_1_check = TestCheck("resource_1", id="CKV_T_1")
        resource_2_check1 = TestCheck("resource_2", id="CKV_T_2")
        resource_2_check2 = TestCheck("resource_2", id="CKV_T_3")
        registry.register(resource_1_check)
        registry.register(resource_2_check1)
        registry.register(resource_2_check2)

        resource_1_checks = registry.get_checks("resource_1")
        self.assertEqual(1, len(resource_1_checks))
        self.assertEqual(resource_1_check, resource_1_checks[0])

        resource_2_checks = registry.get_checks("resource_2")
        self.assertEqual(2, len(resource_2_checks))
        self.assertIn(resource_2_check1, resource_2_checks)
        self.assertIn(resource_2_check2, resource_2_checks)

        self.assertEqual(0, len(registry.get_checks("resource")))
        self.assertEqual(0, len(registry.get_checks("resource_10")))

    def test_get_check_wildcard(self):
        registry = BaseCheckRegistry('')
        resource_s_check = TestCheck("resource_*", id="CKV_T_1")
        resource_as_check = TestCheck("resource_a*", id="CKV_T_2")
        s_check = TestCheck("*", id="CKV_T_3")
        s_2_check = TestCheck("*_2", id="CKV_T_4")
        registry.register(resource_s_check)
        registry.register(resource_as_check)
        registry.register(s_check)
        registry.register(s_2_check)

        resource_1_checks = registry.get_checks("resource_1")
        self.assertEqual(2, len(resource_1_checks))
        self.assertIn(s_check, resource_1_checks)
        self.assertIn(resource_s_check, resource_1_checks)

        resource_2_checks = registry.get_checks("resource_2")
        self.assertEqual(3, len(resource_2_checks))
        self.assertIn(s_check, resource_2_checks)
        self.assertIn(s_2_check, resource_2_checks)
        self.assertIn(resource_s_check, resource_2_checks)

        resource__checks = registry.get_checks("resource_")
        self.assertEqual(2, len(resource__checks))
        self.assertIn(s_check, resource__checks)
        self.assertIn(resource_s_check, resource__checks)

        resource_abc_checks = registry.get_checks("resource_abc")
        self.assertEqual(3, len(resource_abc_checks))
        self.assertIn(s_check, resource_abc_checks)
        self.assertIn(resource_s_check, resource_abc_checks)
        self.assertIn(resource_as_check, resource_abc_checks)

        r_checks = registry.get_checks("r")
        self.assertEqual(1, len(r_checks))
        self.assertIn(s_check, r_checks)

        resource_checks = registry.get_checks("resource")
        self.assertEqual(1, len(resource_checks))
        self.assertIn(s_check, resource_checks)

        resource_checks = registry.get_checks("resource_ABC")
        self.assertEqual(2, len(resource_checks))
        self.assertIn(s_check, resource_checks)
        self.assertIn(resource_s_check, resource_checks)

    def test_get_check_mixed(self):
        registry = BaseCheckRegistry('')
        resource_1_check = TestCheck("resource_1", id="CKV_T_1")
        resource_2_check = TestCheck("resource_2", id="CKV_T_2")
        resource_s_check = TestCheck("resource_*", id="CKV_T_4")
        resource_as_check = TestCheck("resource_a*", id="CKV_T_3")
        s_check = TestCheck("*", id="CKV_T_4")
        s_2_check = TestCheck("*_2", id="CKV_T_5")
        registry.register(resource_1_check)
        registry.register(resource_2_check)
        registry.register(resource_s_check)
        registry.register(resource_as_check)
        registry.register(s_check)
        registry.register(s_2_check)

        resource_1_checks = registry.get_checks("resource_1")
        self.assertEqual(3, len(resource_1_checks))
        self.assertIn(s_check, resource_1_checks)
        self.assertIn(resource_1_check, resource_1_checks)
        self.assertIn(resource_s_check, resource_1_checks)

        resource_10_checks = registry.get_checks("resource_10")
        self.assertEqual(2, len(resource_10_checks))
        self.assertIn(s_check, resource_10_checks)
        self.assertIn(resource_s_check, resource_10_checks)

        resource_2_checks = registry.get_checks("resource_2")
        self.assertEqual(4, len(resource_2_checks))
        self.assertIn(s_check, resource_2_checks)
        self.assertIn(s_2_check, resource_2_checks)
        self.assertIn(resource_2_check, resource_2_checks)
        self.assertIn(resource_s_check, resource_2_checks)

        resource__checks = registry.get_checks("resource_")
        self.assertEqual(2, len(resource__checks))
        self.assertIn(s_check, resource__checks)
        self.assertIn(resource_s_check, resource__checks)


if __name__ == '__main__':
    unittest.main()



from collections import Counter
from pathlib import Path

from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.graph.graph_builder.graph_components.block_types import BlockType
from checkov.github_actions.graph_builder.graph_components.resource_types import ResourceType
from checkov.github_actions.graph_builder.local_graph import GitHubActionsLocalGraph
from checkov.github_actions.runner import Runner

RESOURCES_DIR = Path(__file__).parent.parent / "resources"


def test_build_graph():
    # given
    test_file = str(RESOURCES_DIR / ".github/workflows/supply_chain.yaml")
    template, _ = Runner()._parse_file(f=test_file)
    local_graph = GitHubActionsLocalGraph(definitions={test_file: template})

    # when
    local_graph.build_graph(render_variables=False)

    # then
    assert len(local_graph.vertices) == 6
    assert len(local_graph.edges) == 2

    assert len(local_graph.vertices_by_block_type[BlockType.RESOURCE]) == 6

    job_ids = [vertex.id for vertex in local_graph.vertices if vertex.attributes.get(CustomAttributes.RESOURCE_TYPE) == ResourceType.JOBS]
    step_ids = [vertex.id for vertex in local_graph.vertices if vertex.attributes.get(CustomAttributes.RESOURCE_TYPE) == ResourceType.STEPS]
    permission_ids = [vertex.id for vertex in local_graph.vertices if vertex.attributes.get(CustomAttributes.RESOURCE_TYPE) == ResourceType.PERMISSIONS]
    assert job_ids == ["jobs.bridgecrew", "jobs.bridgecrew2"]
    assert step_ids == ["jobs.bridgecrew.steps.1", "jobs.bridgecrew2.steps.1"]
    assert permission_ids == ["permissions"]

    out_edge_counts = Counter([e.origin for e in local_graph.edges])
    in_edge_counts = Counter([e.dest for e in local_graph.edges])

    assert out_edge_counts == Counter({0: 1, 1: 1})
    assert in_edge_counts == Counter({2: 1, 3: 1})



import unittest
from pathlib import Path

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.severities import Severities, BcSeverities
from checkov.github_actions.runner import Runner
from checkov.runner_filter import RunnerFilter
from checkov.github_actions.checks.registry import registry


class TestRunnerValid(unittest.TestCase):

    def test_registry_has_type(self):
        self.assertEqual(registry.report_type, CheckType.GITHUB_ACTIONS)

    def test_runner(self):
        # given
        test_dir = Path(__file__).parent / "resources"
        checks = ["CKV_GHA_1", "CKV_GHA_2"]

        # when
        report = Runner().run(
            root_folder=str(test_dir), runner_filter=RunnerFilter(framework=["github_actions"], checks=checks)
        )

        # then
        self.assertEqual(len(report.failed_checks), 9)
        self.assertEqual(len(report.parsing_errors), 0)
        self.assertEqual(len(report.passed_checks), 157)
        self.assertEqual(len(report.skipped_checks), 0)

    def test_runner_multi_file(self):
        # given
        file_path = Path(__file__).parent / "gha/.github/workflows/multi_file.yaml"
        file_dir = [str(file_path)]
        filter = RunnerFilter(framework=['github_actions'])

        # when
        report = Runner().run(files=file_dir, runner_filter=filter)

        # then
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.skipped_checks), 0)
        self.assertEqual(len(report.parsing_errors), 0)

    def test_runner_honors_enforcement_rules(self):
        # given
        test_dir = Path(__file__).parent / "resources"
        filter = RunnerFilter(framework=['github_actions'], use_enforcement_rules=True)
        # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
        # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
        filter.enforcement_rule_configs = {CheckType.GITHUB_ACTIONS: Severities[BcSeverities.OFF]}

        # when
        report = Runner().run(
            root_folder=str(test_dir), runner_filter=filter
        )

        # then
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.skipped_checks), 0)
        self.assertEqual(len(report.parsing_errors), 0)

    def test_runner_on_suspectcurl(self):
        # given
        file_path = Path(__file__).parent / "resources/.github/workflows/suspectcurl.yaml"
        file_dir = [str(file_path)]
        checks = ["CKV_GHA_1", "CKV_GHA_3"]

        # when
        report = Runner().run(
            files=file_dir, runner_filter=RunnerFilter(framework=["github_actions"], checks=checks)
        )

        # then
        assert report.failed_checks[0].job[0] == 'prep'
        assert report.failed_checks[0].triggers[0] == {'push', 'workflow_dispatch'}
        assert report.failed_checks[0].workflow_name == 'CI'

        assert report.failed_checks[1].job[0] == 'build'
        assert report.failed_checks[1].triggers[0] == {'push', 'workflow_dispatch'}
        assert report.failed_checks[1].workflow_name == 'CI'

    def test_runner_on_bad_jobs(self):
        # given
        file_path = Path(__file__).parent / "resources/.github/workflows/nested_jobs.yaml"
        file_dir = [str(file_path)]

        # when
        report = Runner().run(
            files=file_dir, runner_filter=RunnerFilter(framework=["github_actions"])
        )

        # then
        assert len(report.passed_checks) + len(report.failed_checks) == 0

    def test_runner_on_shell_injection(self):
        # given
        file_path = Path(__file__).parent / "resources/.github/workflows/shell_injection.yaml"
        file_dir = [str(file_path)]
        checks = ["CKV_GHA_1", "CKV_GHA_3"]

        # when
        report = Runner().run(
            files=file_dir, runner_filter=RunnerFilter(framework=["github_actions"], checks=checks)
        )

        # then
        assert report.passed_checks[0].job[0] == 'unsecure-job'
        assert report.passed_checks[0].triggers[0] == {'issues'}
        assert report.passed_checks[0].workflow_name == 'unsec33ure-worfklow'

        assert report.passed_checks[1].job[0] == 'secure-job'
        assert report.passed_checks[1].triggers[0] == {'issues'}
        assert report.passed_checks[1].workflow_name == 'unsec33ure-worfklow'

        assert report.passed_checks[2].job[0] == 'unsecure-steps'
        assert report.passed_checks[2].triggers[0] == {'issues'}
        assert report.passed_checks[2].workflow_name == 'unsec33ure-worfklow'

    def test_runner_on_netcatreverseshell(self):
        # given
        file_path = Path(__file__).parent / "resources/.github/workflows/netcatreverseshell.yaml"
        file_dir = [str(file_path)]
        checks = ["CKV_GHA_1", "CKV_GHA_3"]

        # when
        report = Runner().run(
            files=file_dir, runner_filter=RunnerFilter(framework=["github_actions"], checks=checks)
        )

        # then
        assert report.passed_checks[0].job[0] == 'prep'
        assert report.passed_checks[0].triggers[0] == {'workflow_dispatch', 'push'}
        assert report.passed_checks[0].workflow_name == 'REVERSESHELL'

        assert report.passed_checks[1].job[0] == 'build'
        assert report.passed_checks[1].triggers[0] == {'workflow_dispatch', 'push'}
        assert report.passed_checks[1].workflow_name == 'REVERSESHELL'

    def test_runner_on_unsecure_command(self):
        # given
        file_path = Path(__file__).parent / "resources/.github/workflows/unsecure_command.yaml"
        file_dir = [str(file_path)]

        checks = ["CKV_GHA_1", "CKV_GHA_3"]

        # when
        report = Runner().run(
            files=file_dir, runner_filter=RunnerFilter(framework=["github_actions"], checks=checks)
        )

        # then
        assert report.failed_checks[0].job[0] == 'unsecure-job'
        assert report.failed_checks[0].triggers[0] == {'pull_request'}
        assert report.failed_checks[0].workflow_name == 'unsecure-worfklow'

        assert report.passed_checks[2].job[0] == 'secure-job'
        assert report.passed_checks[2].triggers[0] == {'pull_request'}
        assert report.passed_checks[2].workflow_name == 'unsecure-worfklow'

    def test_runner_on_non_empty_workflow_dispatch(self):
        # given
        file_path = Path(__file__).parent / "resources/.github/workflows/workflow_dispatch.yaml"
        file_dir = [str(file_path)]

        checks = ["CKV_GHA_7"]

        # when
        report = Runner().run(
            files=file_dir, runner_filter=RunnerFilter(framework=["github_actions"], checks=checks)
        )

        # then
        assert report.failed_checks[0].job[0] == ''
        assert report.failed_checks[0].triggers[0] == {'workflow_dispatch'}
        assert report.failed_checks[0].workflow_name == ''

    def test_runner_on_list_typed_workflow_dispatch(self):
        # given
        file_path = Path(__file__).parent / "resources/.github/workflows/list_workflow_dispatch.yml"
        file_dir = [str(file_path)]

        checks = ["CKV_GHA_7"]

        # when
        report = Runner().run(
            files=file_dir, runner_filter=RunnerFilter(framework=["github_actions"], checks=checks)
        )

        # then
        assert len(report.failed_checks) == 0

    def test_runner_on_supply_chain(self):
        # given
        file_path = Path(__file__).parent / "resources/.github/workflows/supply_chain.yaml"
        file_dir = [str(file_path)]
        checks = ["CKV_GHA_1", "CKV_GHA_3"]

        # when
        report = Runner().run(
            files=file_dir, runner_filter=RunnerFilter(framework=["github_actions"], checks=checks)
        )

        # then
        assert report.failed_checks[0].job[0] == "bridgecrew"
        assert report.failed_checks[0].triggers[0] == {"workflow_dispatch", "schedule"}
        assert report.failed_checks[0].workflow_name == 'Supply Chain'

        assert report.passed_checks[1].job[0] == "bridgecrew2"
        assert report.passed_checks[1].triggers[0] == {"workflow_dispatch", "schedule"}
        assert report.passed_checks[1].workflow_name == 'Supply Chain'

    def test_runner_on_build(self):
        # given
        file_path = Path(__file__).parent.parent.parent / ".github/workflows/build.yml"
        file_dir = [str(file_path)]
        checks = ["CKV_GHA_1", "CKV_GHA_3"]

        # when
        report = Runner().run(
            files=file_dir, runner_filter=RunnerFilter(framework=["github_actions"], checks=checks)
        )

        # then
        assert report.failed_checks[0].job[0] == 'update-bridgecrew-projects'
        assert report.failed_checks[0].triggers[0] == {'workflow_dispatch', 'push'}
        assert report.failed_checks[0].workflow_name == 'build'

        assert report.passed_checks[8].job[0] == "publish-checkov-dockerhub"
        assert report.passed_checks[8].triggers[0] == {'workflow_dispatch', 'push'}
        assert report.passed_checks[8].workflow_name == 'build'

    def test_runner_on_codeql_analysis(self):
        # given
        file_path = Path(__file__).parent.parent.parent / ".github/workflows/codeql-analysis.yml"
        file_dir = [str(file_path)]
        checks = ["CKV_GHA_1", "CKV_GHA_3"]

        # when
        report = Runner().run(
            files=file_dir, runner_filter=RunnerFilter(framework=["github_actions"], checks=checks)
        )

        # then
        assert report.passed_checks[0].job[0] == "analyze"
        assert report.passed_checks[0].triggers[0] == {'push', 'schedule', 'pull_request', 'workflow_dispatch'}
        assert report.passed_checks[0].workflow_name == 'CodeQL'

    def test_runner_on_suspectcurl(self):
        # given
        file_path = Path(__file__).parent / "resources/.github/workflows/empty_jobs.yaml"
        file_dir = [str(file_path)]
        checks = ["CKV_GHA_6", "CKV_GHA_5"]

        # when
        report = Runner().run(
            files=file_dir, runner_filter=RunnerFilter(framework=["github_actions"], checks=checks)
        )

        # then
        assert len(report.failed_checks) == 0
        assert len(report.passed_checks) == 0
        assert len(report.skipped_checks) == 0
        assert len(report.parsing_errors) == 0

    def test_runner_on_permissions(self):
        # given
        file_path = Path(__file__).parent / "gha/.github/workflows/failed.yaml"
        file_dir = [str(file_path)]
        checks = ["CKV2_GHA_1"]

        # when
        report = Runner().run(
            files=file_dir, runner_filter=RunnerFilter(framework=["github_actions"], checks=checks)
        )

        # then
        assert len(report.failed_checks) == 1
        assert report.failed_checks[0].file_line_range == [7, 8]
        assert len(report.passed_checks) == 0
        assert len(report.skipped_checks) == 0
        assert len(report.parsing_errors) == 0

    def test_runner_on_workflows_dispatch(self):
        # given
        file_path = Path(__file__).parent / "gha/.github/workflows/bad_workflows_dispatch.yaml"
        file_dir = [str(file_path)]
        checks = ["CKV_GHA_7"]

        # when
        report = Runner().run(
            files=file_dir, runner_filter=RunnerFilter(framework=["github_actions"], checks=checks)
        )

        # then
        assert len(report.failed_checks) == 1
        assert len(report.passed_checks) == 0
        assert len(report.skipped_checks) == 0
        assert len(report.parsing_errors) == 0


if __name__ == "__main__":
    unittest.main()



from datetime import datetime, timezone

from time_machine import travel

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.output.extra_resource import ExtraResource
from checkov.common.output.report import Report
from checkov.common.output.spdx import SPDX
from checkov.common.sca.output import create_report_cve_record


@travel(datetime(2022, 12, 24, tzinfo=timezone.utc))
def test_sca_package_output():
    # given
    rootless_file_path = "requirements.txt"
    file_abs_path = "/path/to/requirements.txt"
    check_class = "checkov.sca_package_2.scanner.Scanner"
    vulnerability_details = {
        "id": "CVE-2019-19844",
        "status": "fixed in 3.0.1, 2.2.9, 1.11.27",
        "cvss": 9.8,
        "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
        "description": "Django before 1.11.27, 2.x before 2.2.9, and 3.x before 3.0.1 allows account takeover. ...",
        "severity": "moderate",
        "packageName": "django",
        "packageVersion": "1.2",
        "link": "https://nvd.nist.gov/vuln/detail/CVE-2019-19844",
        "riskFactors": ["Attack complexity: low", "Attack vector: network", "Critical severity", "Has fix"],
        "impactedVersions": ["<1.11.27"],
        "publishedDate": "2019-12-18T20:15:00+01:00",
        "discoveredDate": "2019-12-18T19:15:00Z",
        "fixDate": "2019-12-18T20:15:00+01:00",
    }

    record = create_report_cve_record(
        rootless_file_path=rootless_file_path,
        file_abs_path=file_abs_path,
        check_class=check_class,
        vulnerability_details=vulnerability_details,
        licenses="OSI_BDS",
        package={"package_registry": "https://registry.npmjs.org/", "is_private_registry": False},
    )
    # also add a BC_VUL_2 record
    bc_record = create_report_cve_record(
        rootless_file_path=rootless_file_path,
        file_abs_path=file_abs_path,
        check_class=check_class,
        vulnerability_details=vulnerability_details,
        licenses="OSI_BDS",
        package={"package_registry": "https://registry.npmjs.org/", "is_private_registry": False},
    )
    bc_record.check_id = "BC_VUL_2"

    report = Report(CheckType.SCA_PACKAGE)
    report.add_resource(record.resource)
    report.add_record(record)
    report.add_record(bc_record)

    report.extra_resources.add(
        ExtraResource(
            file_abs_path=file_abs_path,
            file_path=f"/{rootless_file_path}",
            resource=f"{rootless_file_path}.testpkg",
            vulnerability_details={"package_name": "testpkg", "package_version": "1.1.1", "licenses": "MIT"},
        )
    )

    # when
    spdx = SPDX(repo_id="example", reports=[report])

    # override dynamic data
    spdx.document.creation_info.document_namespace = "https://spdx.org/spdxdocs/checkov-sbom-9.9.9"

    # then
    output = spdx.get_tag_value_output()

    # remove dynamic data
    assert output == "".join(
        [
            "## Document Information\n",
            "SPDXVersion: SPDX-2.3\n",
            "DataLicense: CC0-1.0\n",
            "SPDXID: SPDXRef-DOCUMENT\n",
            "DocumentName: checkov-sbom\n",
            "DocumentNamespace: https://spdx.org/spdxdocs/checkov-sbom-9.9.9\n",
            "\n",
            "## Creation Information\n",
            "Creator: Tool: checkov\n",
            "Creator: Organization: bridgecrew (meet@bridgecrew.io)\n",
            "Created: 2022-12-24T00:00:00Z\n",
            "\n",
            "## Package Information\n",
            "PackageName: django\n",
            "SPDXID: SPDXRef-django\n",
            "PackageVersion: 1.2\n",
            "PackageFileName: /requirements.txt\n",
            "PackageDownloadLocation: NONE\n",
            "FilesAnalyzed: true\n",
            "PackageLicenseInfoFromFiles: OSI_BDS\n",
            "\n",
            "## Package Information\n",
            "PackageName: testpkg\n",
            "SPDXID: SPDXRef-testpkg\n",
            "PackageVersion: 1.1.1\n",
            "PackageFileName: /requirements.txt\n",
            "PackageDownloadLocation: NONE\n",
            "FilesAnalyzed: true\n",
            "PackageLicenseInfoFromFiles: MIT\n",
            "\n",
            "\n",
        ]
    )



from __future__ import annotations

from textwrap import dedent

import pytest
from _pytest.capture import CaptureFixture

from checkov.common.graph.checks_infra.debug import attribute_block, graph_check, connection_block
from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.util.env_vars_config import env_vars_config


@pytest.fixture
def enable_graph_debug():
    env_vars_config.EXPERIMENTAL_GRAPH_DEBUG = True
    yield
    env_vars_config.EXPERIMENTAL_GRAPH_DEBUG = False


def test_no_output_on_default(capfd: CaptureFixture[str]):
    # given/when
    graph_check(check_id="CKV_EXAMPLE_1", check_name="Example")

    # then
    assert not capfd.readouterr().out


def test_attribute_block(capfd: CaptureFixture[str], enable_graph_debug):
    # given
    resource_types = ["aws_s3_bucket"]
    attribute = "lifecycle_rule"
    operator = Operators.EXISTS
    value = None
    resource = {
        CustomAttributes.ID: "aws_s3_bucket.example",
        CustomAttributes.CONFIG: {
            "aws_s3_bucket": {
                "example": {
                    "__end_line__": 3,
                    "__start_line__": 1,
                    "bucket": ["example"],
                    "__address__": "aws_s3_bucket.example",
                }
            }
        },
    }
    status = "failed"

    # when
    attribute_block(
        resource_types=resource_types,
        attribute=attribute,
        operator=operator,
        value=value,
        resource=resource,
        status=status,
    )

    # then
    assert capfd.readouterr().out == dedent(
        """
        Evaluated block:
        
        - cond_type: attribute
          resource_types:
          - aws_s3_bucket
          attribute: lifecycle_rule
          operator: exists
        
        and got:
        
        Resource "aws_s3_bucket.example" failed:
        {
          "aws_s3_bucket": {
            "example": {
              "__end_line__": 3,
              "__start_line__": 1,
              "bucket": [
                "example"
              ],
              "__address__": "aws_s3_bucket.example"
            }
          }
        }
        """
    )


def test_connection_block(capfd: CaptureFixture[str], enable_graph_debug):
    # given
    resource_types = ["aws_s3_bucket"]
    connected_resource_types = ["aws_s3_bucket_lifecycle_configuration"]
    operator = Operators.EXISTS
    value = None
    passed_resources = [
        {
            CustomAttributes.ID: "aws_s3_bucket.good",
        },
        {
            CustomAttributes.ID: "aws_s3_bucket_lifecycle_configuration.good",
        },
    ]
    failed_resources = [
        {
            CustomAttributes.ID: "aws_s3_bucket.bad",
        }
    ]

    # when
    connection_block(
        resource_types=resource_types,
        connected_resource_types=connected_resource_types,
        operator=operator,
        passed_resources=passed_resources,
        failed_resources=failed_resources,
    )

    # then
    assert capfd.readouterr().out == dedent(
        """
        Evaluated blocks:
        
        - cond_type: connection
          resource_types:
          - aws_s3_bucket
          connected_resource_types:
          - aws_s3_bucket_lifecycle_configuration
          operator: exists
        
        and got:
        
        Passed resources: "aws_s3_bucket.good", "aws_s3_bucket_lifecycle_configuration.good"
        Failed resources: "aws_s3_bucket.bad"
        """
    )



from aws_cdk import core
from aws_cdk import aws_rds as rds

class RDSStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create an RDS DB instance with PubliclyAccessible set to True
        rds_instance = rds.DatabaseInstance(
            self,
            "MyRDSInstance",
            engine=rds.DatabaseInstanceEngine.mysql(
                version=rds.MysqlEngineVersion.VER_8_0
            ),
            instance_type=core.Fn.select(0, core.Fn.split(" ", "db.m5.large")),
            publicly_accessible=True,  # Set PubliclyAccessible to True
            allocated_storage=20,
            max_allocated_storage=100,
            vpc_subnets={
                "subnetType": core.Fn.select(0, core.Fn.split(",", "private")),
            },
            storage_type=rds.StorageType.GP2,
            removal_policy=core.RemovalPolicy.DESTROY,
        )

app = core.App()
RDSStack(app, "RDSStack")
app.synth()



import unittest
import os

from checkov.terraform.checks.resource.panos.NetworkIPsecAlgorithms import check
from checkov.runner_filter import RunnerFilter
from checkov.terraform.runner import Runner


class NetworkIPsecAlgorithms(unittest.TestCase):

    def test(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files_dir = current_dir + "/example_NetworkIPsecAlgorithms"
        report = runner.run(root_folder=test_files_dir, runner_filter=RunnerFilter(checks=[check.id]))
        summary = report.get_summary()

        passing_resources = {
            'panos_ipsec_crypto_profile.pass1',
            'panos_panorama_ipsec_crypto_profile.pass2',
            'panos_ipsec_crypto_profile.pass3',
            'panos_panorama_ipsec_crypto_profile.pass4',
            'panos_ipsec_crypto_profile.pass5',
            'panos_panorama_ipsec_crypto_profile.pass6',
        }
        failing_resources = {
            'panos_ipsec_crypto_profile.fail1',
            'panos_ipsec_crypto_profile.fail2',
            'panos_ipsec_crypto_profile.fail3',
            'panos_ipsec_crypto_profile.fail4',
            'panos_ipsec_crypto_profile.fail5',
            'panos_ipsec_crypto_profile.fail6',
            'panos_panorama_ipsec_crypto_profile.fail7',
            'panos_panorama_ipsec_crypto_profile.fail8',
            'panos_panorama_ipsec_crypto_profile.fail9',
            'panos_panorama_ipsec_crypto_profile.fail10',
            'panos_panorama_ipsec_crypto_profile.fail11',
            'panos_panorama_ipsec_crypto_profile.fail12',
            'panos_ipsec_crypto_profile.fail13',
            'panos_ipsec_crypto_profile.fail14',
            'panos_panorama_ipsec_crypto_profile.fail15',
            'panos_panorama_ipsec_crypto_profile.fail16',
        }

        passed_check_resources = set([c.resource for c in report.passed_checks])
        failed_check_resources = set([c.resource for c in report.failed_checks])

        self.assertEqual(summary['passed'], 6)
        self.assertEqual(summary['failed'], 16)
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)


if __name__ == '__main__':
    unittest.main()



import unittest
from pathlib import Path

from checkov.kubernetes.parser.k8_yaml import load

EXAMPLES_DIR = Path(__file__).parent / "examples"


class TestScannerRegistry(unittest.TestCase):
    def test_load_pod(self):
        # given
        file_path = EXAMPLES_DIR / "yaml/busybox.yaml"

        # when
        template, file_lines = load(file_path)

        # then
        assert len(template) == 1
        assert template[0]["apiVersion"] == "v1"
        assert template[0]["kind"] == "Pod"
        assert len(file_lines) == 28


    def test_load_not_k8s_file(self):
        # given
        file_path = EXAMPLES_DIR / "yaml/normal.yaml"

        # when
        template, file_lines = load(file_path)

        # then
        assert template == [{}]
        assert file_lines == []


    def test_load_helm_template_file(self):
        # given
        file_path = EXAMPLES_DIR / "yaml/helm.yaml"

        # when
        template, file_lines = load(file_path)

        # then
        assert template == [{}]
        assert file_lines == []

    def test_load_helm_vars_file(self):
        # given
        file_path = EXAMPLES_DIR / "yaml/helm2.yaml"

        # when
        template, file_lines = load(file_path)

        # then
        assert template == [{}]
        assert file_lines == []

    def test_load_utf8_bom_file(self):
        # given
        file_path = EXAMPLES_DIR / "yaml/busybox_utf8_bom.yaml"

        # when
        template, file_lines = load(file_path)

        # then
        assert len(template) == 1
        assert template[0]["apiVersion"] == "v1"
        assert template[0]["kind"] == "Pod"
        assert len(file_lines) == 28

    def test_load_templating_configmap(self):
        # given
        file_path = EXAMPLES_DIR / "yaml/not_helm_configmap.yaml"

        # when
        template, file_lines = load(file_path)

        # then
        assert len(template) == 1
        assert template[0]["apiVersion"] == "v1"
        assert template[0]["kind"] == "ConfigMap"
        assert len(file_lines) == 8

if __name__ == '__main__':
    unittest.main()



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.output.report import Report
from checkov.common.util.type_forcers import force_dict
from checkov.gitlab_ci.checks.registry import registry
from checkov.gitlab_ci.common.resource_id_utils import generate_resource_key_recursive
from checkov.runner_filter import RunnerFilter
from checkov.yaml_doc.runner import Runner as YamlRunner

if TYPE_CHECKING:
    from checkov.common.checks.base_check_registry import BaseCheckRegistry
    from collections.abc import Iterable


class Runner(YamlRunner):
    check_type = CheckType.GITLAB_CI  # noqa: CCE003  # a static attribute

    def require_external_checks(self) -> bool:
        return False

    def import_registry(self) -> BaseCheckRegistry:
        return registry

    @staticmethod
    def _parse_file(
        f: str, file_content: str | None = None
    ) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
        if Runner.is_workflow_file(f):
            return YamlRunner._parse_file(f=f, file_content=file_content)

        return None

    @staticmethod
    def is_workflow_file(file_path: str) -> bool:
        """
        :return: True if the file mentioned is in the gitlab workflow name .gitlab-ci.yml. Otherwise: False
        """
        return file_path.endswith((".gitlab-ci.yml", ".gitlab-ci.yaml"))

    def included_paths(self) -> Iterable[str]:
        return (".gitlab-ci.yml", ".gitlab-ci.yaml")

    def get_resource(self, file_path: str, key: str, supported_entities: Iterable[str],
                     start_line: int = -1, end_line: int = -1, graph_resource: bool = False) -> str:
        file_config = force_dict(self.definitions[file_path])
        if not file_config:
            return key
        resource_id: str = generate_resource_key_recursive(conf=file_config, key='', start_line=start_line,
                                                           end_line=end_line)
        return resource_id

    def run(
            self,
            root_folder: str | None = None,
            external_checks_dir: list[str] | None = None,
            files: list[str] | None = None,
            runner_filter: RunnerFilter | None = None,
            collect_skip_comments: bool = True,
    ) -> Report | list[Report]:
        runner_filter = runner_filter or RunnerFilter()
        report = super().run(root_folder=root_folder, external_checks_dir=external_checks_dir,
                             files=files, runner_filter=runner_filter, collect_skip_comments=collect_skip_comments)
        return report



import pytest

from checkov.terraform.tag_providers import get_provider_tag


@pytest.mark.parametrize("resource_type, expected", [
    ("aws_instance.example", "aws"),
    ("module.test.aws_instance.example", "aws"),
    ("azure_instance.example", "azure"),
    ("google_instance.example", "gcp"),
])
def test_get_provider_tag(resource_type, expected) -> None:
    assert get_provider_tag(resource_type) == expected



from typing import TextIO

from typing_extensions import TypedDict


class _Instruction(TypedDict):
    instruction: str
    startline: int
    endline: int
    content: str
    value: str


class DockerfileParser:
    def __init__(
        self,
        path: str | None = ...,
        cache_content: bool = ...,
        env_replace: bool = ...,
        parent_env: dict[str, str] | None = None,
        fileobj: TextIO | None = None,
        build_args: dict[str, str] | None = None,
    ) -> None: ...

    @property
    def lines(self) -> list[str]: ...

    @property
    def structure(self) -> list[_Instruction]: ...



import pytest
from checkov.common.models.enums import CheckResult

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.output.secrets_record import SecretsRecord

from checkov.common.output.report import Report

kwargs = {'validation_status': 'mock', 'check_id': 'mock', 'check_name': 'mock', 'code_block': 'mock',
          'file_path': 'mock',
          'file_line_range': 'mock', 'evaluations': 'mock', 'check_class': 'mock', 'file_abs_path': 'mock'}


@pytest.fixture
def report() -> Report:
    record_1 = SecretsRecord(bc_check_id="VIOLATION_1", resource="RESOURCE_1",
                             check_result={"result": CheckResult.FAILED}, **kwargs)
    record_2 = SecretsRecord(bc_check_id="VIOLATION_2", resource="RESOURCE_2",
                             check_result={"result": CheckResult.FAILED}, **kwargs)

    record_3 = SecretsRecord(bc_check_id="VIOLATION_3", resource="RESOURCE_1",
                             check_result={"result": CheckResult.PASSED}, **kwargs)

    record_4 = SecretsRecord(bc_check_id="VIOLATION_1", resource="RESOURCE_2",
                             check_result={"result": CheckResult.SKIPPED}, **kwargs)

    report = Report(CheckType.SECRETS)
    report.add_record(record_1)
    report.add_record(record_2)
    report.add_record(record_3)
    report.add_record(record_4)

    return report



from pathlib import Path

import pytest

from checkov.arm.graph_manager import ArmGraphManager
from checkov.arm.utils import get_files_definitions, ArmElements
from checkov.arm.graph_builder.graph_components.block_types import BlockType
from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.db_connectors.rustworkx.rustworkx_db_connector import RustworkxConnector
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.graph.graph_builder.consts import GraphSource
from checkov.common.util.consts import START_LINE, END_LINE

EXAMPLES_DIR = Path(__file__).parent / "examples"


@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector
    ]
)
def test_build_graph_from_definitions(graph_connector):
    # given
    test_file = str(EXAMPLES_DIR / "container_instance.json")
    definitions, _, _ = get_files_definitions([test_file])

    graph_manager = ArmGraphManager(db_connector=graph_connector())

    # when
    local_graph = graph_manager.build_graph_from_definitions(definitions=definitions, render_variables=False)

    # then
    assert len(local_graph.vertices) == 18
    assert len(local_graph.edges) == 24

    # resource name will change, when variable rendering is supported
    container_idx = local_graph.vertices_by_path_and_id[(test_file, "Microsoft.ContainerInstance/containerGroups.[parameters('containerGroupName')]")]
    container = local_graph.vertices[container_idx]

    assert container.block_type == BlockType.RESOURCE
    assert container.id == "Microsoft.ContainerInstance/containerGroups.[parameters('containerGroupName')]"
    assert container.source == GraphSource.ARM

    assert container.attributes[START_LINE] == 156
    assert container.attributes[END_LINE] == 191
    assert container.attributes[CustomAttributes.RESOURCE_TYPE] == "Microsoft.ContainerInstance/containerGroups"

    assert container.config["type"] == "Microsoft.ContainerInstance/containerGroups"
    assert container.config["apiVersion"] == "2019-12-01"
    assert container.config["name"] == "[parameters('containerGroupName')]"
    assert container.config["location"] == "[parameters('location')]"

    assert container.config["properties"]["osType"] == "Linux"
    assert container.config["properties"]["restartPolicy"] == "Always"


@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector
    ]
)
def test_build_graph_from_definitions2(graph_connector):
    # given
    test_file = str(EXAMPLES_DIR / "container_instance.json")
    definitions, _, _ = get_files_definitions([test_file])

    graph_manager = ArmGraphManager(db_connector=graph_connector())

    # when
    local_graph = graph_manager.build_graph_from_definitions(definitions=definitions)

    # then
    assert len(local_graph.vertices) == 18
    assert len(local_graph.edges) == 27 # more edges because we did render_variables and its add the dependsOn edges


import unittest

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.aws.GlueDataCatalogEncryption import check
import hcl2

class TestGlueDataCatalogEncryption(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
resource "aws_glue_data_catalog_encryption_settings" "test" {
  data_catalog_encryption_settings {
    connection_password_encryption {
      return_connection_password_encrypted = false
    }
    encryption_at_rest {
      catalog_encryption_mode = "DISABLED"
    }
  }
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_glue_data_catalog_encryption_settings']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
resource "aws_glue_data_catalog_encryption_settings" "test" {
  data_catalog_encryption_settings {
    connection_password_encryption {
      aws_kms_key_id                       = aws_kms_key.test.arn
      return_connection_password_encrypted = true
    }
    encryption_at_rest {
      catalog_encryption_mode = "SSE-KMS"
      sse_aws_kms_key_id      = aws_kms_key.test.arn
    }
  }
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_glue_data_catalog_encryption_settings']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

if __name__ == '__main__':
    unittest.main()
